//-----------------------------------------------------------------------------------
// File:		log.cpp
//
// Version:	2.0
//
// Desc:		Implementierungsdatei der Log-Klasse.
//
// Copyright:	Open-Source
// Autor:		Johannes Bergmann
// Kontakt:		jobe@fambgm.de
//-----------------------------------------------------------------------------------
#include "clog.h"




//-----------------------------------------------------------------------------------
// Name: CLog
// Desc: Standart-Konstruktor der Log-Klasse; Initialisiert die Klasse nur
//-----------------------------------------------------------------------------------
CLog::CLog()
{
	this->m_bLoaded		= false;
	this->m_fz			= NULL;
}




//-----------------------------------------------------------------------------------
// Name: CLog
// Desc: berladener Konsruktor; Das Logfile kann danach beschrieben werden
//
// Parameters:	1) Dateipfad des zu erstellenden Logfiles
//					2) optional; true = Zeit wird mitprotokolliert
//-----------------------------------------------------------------------------------
CLog::CLog(const char * chFilename)
{
	this->m_bLoaded		= false;
	this->m_fz			= NULL;

	this->Create(chFilename);
}




//-----------------------------------------------------------------------------------
// Name: ~CLog
// Desc: Destruktor
//-----------------------------------------------------------------------------------
CLog::~CLog()
{
	this->Close();
}




//-----------------------------------------------------------------------------------
// Name: Create
// Desc: Erstellt ein Logfile
//
// Parameters: 1) Dateipfad des zu erstellenden Logfiles
//					2) optional; true = Zeit wird mitprotokolliert
//
// Return Value: true bei Erfolg
//-----------------------------------------------------------------------------------
bool CLog::Create(const char * chFilename)
{
	// Ist schon eine Datei geoeffnet?
	if(this->m_bLoaded)
		return false;

	// Datei oeffnen
	if((this->m_fz = fopen(chFilename, "w")) == NULL)
		return false;

	// Datum und Zeit ermitteln
	time(&this->m_Time);
	this->m_curTime = localtime(&this->m_Time);

	// Header in Logfile schreiben
	fprintf(this->m_fz, "#----------------------------------------\r\n");
	fprintf(this->m_fz, "# Logfile opened.\r\n# Date: %i.%i.%i\r\n# Time: %i:%i:%i\r\n",
				this->m_curTime->tm_mday, this->m_curTime->tm_mon + 1,
				this->m_curTime->tm_year + 1900, this->m_curTime->tm_hour,
				this->m_curTime->tm_min, this->m_curTime->tm_sec);
	fprintf(this->m_fz, "#----------------------------------------\r\n");

	// Membervariablen setzen
	this->m_bLoaded	= true;

	return true;
}




//-----------------------------------------------------------------------------------
// Name: Write
// Desc: Schreibt einen Eintrag ins Logfile
//
// Parameters: 1) Tiefe des Eintrags; dient zur Strukturierung
//					2) Unformatierter Text
//					3) Parameter zum Formatieren des Textes
//
// Return Value: true bei Erfolg
//-----------------------------------------------------------------------------------
bool CLog::Write(unsigned short int iDepth, const char * chText, ...)
{
	// Ist die Datei geoeffnet?
	if(!this->m_bLoaded)
		return false;

	// Neue Zeile bei neuem Abschnitt
	if(iDepth == 0)
		fprintf(this->m_fz, "\r\n");

	// Zur Formatierung
	for(int i=0; i<iDepth; i++)
		fprintf(this->m_fz, "  ");
	fputc('-', this->m_fz);

	// Zeit schreiben
	time(&(this->m_Time));
	this->m_curTime = localtime(&(this->m_Time));

	fprintf(this->m_fz, "%i:%i:%i:   ", this->m_curTime->tm_hour,
				this->m_curTime->tm_min, this->m_curTime->tm_sec);
	

	// Eigentlichen Text des Eintrags schreiben
	va_list zeiger;
	va_start(zeiger, chText);
	vfprintf(this->m_fz, chText, zeiger);
	va_end(zeiger);

	// Eine neue Zeile einfuegen
	fprintf(this->m_fz, "\r\n");

	fflush(this->m_fz); //By Z-Man

	return true;
}




//-----------------------------------------------------------------------------------
// Name: Close
// Desc: Schliet das Logfile
//
// Return Value: true bei Erfolg
//-----------------------------------------------------------------------------------
bool CLog::Close()
{
	// Ist die Datei ueberhaupt geoeffnet?
	if(!this->m_bLoaded)
		return false;

	// Datum und Zeit ermitteln
	time(&this->m_Time);
	this->m_curTime = localtime(&this->m_Time);

	// Abschlussinfos schreiben
	fprintf(this->m_fz, "\r\n#----------------------------------------\r\n");
	fprintf(this->m_fz, "# Logfile closed.\r\n# Date: %i.%i.%i\r\n# Time: %i:%i:%i\r\n",
				this->m_curTime->tm_mday, this->m_curTime->tm_mon + 1,
				this->m_curTime->tm_year + 1900, this->m_curTime->tm_hour,
				this->m_curTime->tm_min, this->m_curTime->tm_sec);
	fprintf(this->m_fz, "#----------------------------------------\r\n");

	// Datei schliessen
	fclose(this->m_fz);
	this->m_bLoaded = false;

	return true;
}

//last changes: tijot 0.89.3 - no_skill
