//-----------------------------------------------------------------------------------
// File: fmod.cpp
//
// Desc: Eine Klasse zum Benutzen von FMOD. Sorry, dass der Header auch fmod.h
//			 heisst, aber bin halt schlecht im Namen ausdenken ;-)
//
// Copyright (C) 2003 by Johannes Bergmann <jobe@fambgm.de>
//
// This program is free software; you can redistribute ist and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
// This program is distributed in the hope that it will be useful, but WITHOUT ANY
// WARRANTY; without even the implied Warranty of MERCHANTABILITY or FITNESS FOR A
// PARTICULAR PURPOSE. See the GNU General Public License for more details.
// You should have received a copy of the GNU General Public License along with this
// program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
// Cambridge, MA 02139, USA.
//-----------------------------------------------------------------------------------
#include "csound.h"




//-----------------------------------------------------------------------------------
// Name: CSound
// Desc: Der Konstruktor initialisiert die Membervariablen und ggf. FMod
//-----------------------------------------------------------------------------------
CSound::CSound(const char *chFilename)
{
	// FMOD initialisieren
	if(!g_bAllreadyInitFmod)
	{
		g_bAllreadyInitFmod = true;

		if (FSOUND_GetVersion() < FMOD_VERSION)
    {
        printf("Error : You are using the wrong DLL version!  You should be using FMOD %.02f\n", FMOD_VERSION);
        printf("Game will continue without playing Sounds!\n");
				g_bAllreadyInitFmod = false;
    }

		if (!FSOUND_Init(44100, 64, 0))
		{
			printf("Could not init FMod: %s\n", FMOD_ErrorString(FSOUND_GetError()));
			printf("Game will continue without playing Sounds!\n");
			g_bAllreadyInitFmod = false;
		}

	}

	// ggf. Sound laden
	m_stream = NULL;
	if(chFilename)
		Load(chFilename);

	// Zaehler fuer Instanzen dekrimieren
	g_iInstances++;
}




//-----------------------------------------------------------------------------------
// Name: ~CSound
// Desc: Der Konstruktor gibt den Speicher wieder frei und beenden ggf. FMod
//-----------------------------------------------------------------------------------
CSound::~CSound()
{
	if(m_stream)
	{
		FSOUND_Stream_Stop(m_stream);
		FSOUND_Stream_Close(m_stream);
	}

	g_iInstances--;

	// FMod ggf. beenden
	if(g_iInstances <= 0)
		FSOUND_Close();
}




//-----------------------------------------------------------------------------------
// Name: Load
// Desc: Laed den Sound
//-----------------------------------------------------------------------------------
bool CSound::Load(const char *chFilename)
{
	if(!chFilename)
		return false;

	if(!g_bAllreadyInitFmod)
		return false;

	if(m_stream)
	{
		FSOUND_Stream_Stop(m_stream);
		FSOUND_Stream_Close(m_stream);
		m_stream = NULL;
	}

	if(!(m_stream = FSOUND_Stream_Open(chFilename, FSOUND_NORMAL | FSOUND_MPEGACCURATE | FSOUND_LOOP_NORMAL, 0, 0)))
	{
		printf("Could not load sound \'%s\': %s\n", chFilename, FMOD_ErrorString(FSOUND_GetError()));
		return false;
	}

	if(!FSOUND_Stream_SetEndCallback(m_stream, CSound::StopSoundCB, 0))
		return false;

	return true;
}




//-----------------------------------------------------------------------------------
// Name: Play
// Desc: Spielt den Sound
//-----------------------------------------------------------------------------------
bool CSound::Play(bool bLoop)
{
	if(!m_stream)
		return false;

	if(bLoop)
		FSOUND_Stream_SetLoopCount(m_stream, -1);
	else
		FSOUND_Stream_SetLoopCount(m_stream, 0);

	if(FSOUND_Stream_Play(FSOUND_FREE, m_stream) == -1)
		return false;


	return true;
}




//-----------------------------------------------------------------------------------
// Name: Stop
// Desc: Stoppt den Sound
//-----------------------------------------------------------------------------------
void CSound::Stop(void)
{
	if(m_stream)
		FSOUND_Stream_Stop(m_stream);
}




//-----------------------------------------------------------------------------------
// Name: StopSoundCB
// Desc: Diese Callback-Funktion wird aufgerufen, um einen Sound zu stoppen, wenn er
//			 schon aus ist ;-)
//-----------------------------------------------------------------------------------
#ifndef WIN32
signed char CSound::StopSoundCB(FSOUND_STREAM *stream, void *buff, int len, int param)
#else
signed char __stdcall CSound::StopSoundCB(FSOUND_STREAM *stream,void *buff,int len,int param)
#endif
{
	FSOUND_Stream_Stop(stream);
		return (signed char)true;
}



//last changes: tijot 0.89.3 - no_skill

