#include "tfree.h"


freeMain *g_freemain = NULL;	//saved here, so freeFont, freeSprite and freeAnimsprite don't need an own pointer to the only freemain object


void freeMain::msg(int level, char *s, ...){
	char		buffer[1024];
	va_list		zeiger;
	
	//printf
	va_start(zeiger, s);
	vsprintf(buffer, s, zeiger);
	va_end(zeiger);

	//write to console
	for(int i=1; i< level; i++)
	 putchar(' ');
	puts(buffer);

	//write to logfile
	log.Write(level, buffer);
}


/*
bool freeMain::UpdateMessages(){
	while(SDL_PollEvent(&m_event)){
		if (m_event.type == SDL_QUIT)
			return false;
	}

	return true;
}
*/


freeMain::freeMain(){

}


freeMain::~freeMain(){
	msg(1, "Shutting down Engine");
	msg(2, "Quit SDL");
	SDL_Quit();
	msg(2, "DONE - Quit SDL");
}


bool freeMain::init(int screenwidth, int screenheight, bool fullscreen, const char *logfile){
	if(g_freemain != NULL){
		fprintf(stderr, "there's already a freeMain object!");
		return false;
	}

	if ( log.Create(logfile) == false ){
		fprintf(stderr, "Couldn't create logfile %s", logfile);
		return false;
	}


	
	msg(1, "Initialising Engine");

	msg(2, "init SDL");
    if( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
		msg(3, "Couldn't initialize SDL: %s\n", SDL_GetError());
        return false;
    }


	//Uint32 flags = SDL_GLSDL | SDL_HWSURFACE | SDL_FULLSCREEN | SDL_DOUBLEBUF;
	Uint32 flags = SDL_DOUBLEBUF;
    
	if(fullscreen)
		flags |= SDL_FULLSCREEN;
		
	msg(3, "set video mode %ix%i @ [optimal] bpp, fullscreen: %i", screenwidth, screenheight, fullscreen);
	screen = SDL_SetVideoMode(screenwidth, screenheight, 0, flags);

    if ( screen == NULL ) {
		msg(4, "Couldn't set %dx%dx[optimal] video mode: %s\n", screenwidth, screenheight, SDL_GetError());
        return false;
    }

	const SDL_VideoInfo *vi = SDL_GetVideoInfo();
	msg(4, "running @ %d bpp", vi->vfmt->BitsPerPixel);

	msg(2, "Retrieving Keystate Array");
	keystates = SDL_GetKeyState(NULL);

	msg(2, "DONE - init SDL");
	msg(1, "DONE - Initialising Engine");
	//ok
	g_freemain = this;

	return true;
}




freeSprite::freeSprite(){
	m_bltrect.x = 0;
	m_bltrect.y = 0;
	m_bltrect.w = 0;
	m_bltrect.h = 0;
	m_picture = NULL;
	
}

freeSprite::~freeSprite(){
	// Free the allocated BMP surface 
	//--- it seems like i get an error if i delete the surface retrieved from SDL_DisplayFormat()
	//SDL_FreeSurface(m_picture);
}

bool freeSprite::init(const char *filename, Uint32 transparentcolor){
	g_freemain->msg(1, "loading sprite %s", filename);

    // Load the BMP file into a surface
    m_picture = IMG_Load(filename);

    if (m_picture == NULL) {
		g_freemain->msg(2, "Couldn't load %s: %s\n", filename, SDL_GetError());
        return false;
    }

	if( SDL_SetColorKey(m_picture, SDL_SRCCOLORKEY | SDL_RLEACCEL, transparentcolor) < 0){
		g_freemain->msg(2, "Couldn't set ColorKey + RLE for %s: %s", filename, SDL_GetError());
		return false;
	}

	SDL_Surface *temp;
	if( (temp = SDL_DisplayFormat(m_picture)) == NULL){
		g_freemain->msg(2, "couldn't convert %s to display format: %s", filename, SDL_GetError());
		return false;
	}
	SDL_FreeSurface(m_picture);
	m_picture = temp;


	m_bltrect.w = m_picture->w;
	m_bltrect.h = m_picture->h;

	g_freemain->msg(2, "DONE - loading sprite");
	return true;
}

bool freeSprite::init(const char *filename) {
	g_freemain->msg(1, "loading sprite %s", filename);

    // Load the BMP file into a surface
    m_picture = IMG_Load(filename);

    if (m_picture == NULL) {
		g_freemain->msg(2, "Couldn't load %s: %s\n", filename, SDL_GetError());
        return false;
    }

	SDL_Surface *temp;
	if( (temp = SDL_DisplayFormat(m_picture)) == NULL){
		g_freemain->msg(2, "couldn't convert %s to display format: %s", filename, SDL_GetError());
		return false;
	}
	SDL_FreeSurface(m_picture);
	m_picture = temp;


	m_bltrect.w = m_picture->w;
	m_bltrect.h = m_picture->h;

	g_freemain->msg(2, "DONE - loading sprite");
	return true;
}





bool freeSprite::draw(int x, int y){
	m_bltrect.x = x;
	m_bltrect.y = y;

	// Blit onto the screen surface
	if(SDL_BlitSurface(m_picture, NULL, g_freemain->getScreen(), &m_bltrect) < 0){
		fprintf(stderr, "BlitSurface error: %s\n", SDL_GetError());
		return false;
	}
	

	return true;
}







bool freeAnimSprite::init(const char *filename, Uint32 transparentcolor, int picwidth, int picheight, Uint32 delay){
    // Load the BMP file into a surface
	g_freemain->msg(1, "loading AnimSprite %s", filename);
    m_picture = IMG_Load(filename);

    if (m_picture == NULL) {
        g_freemain->msg(2, "Couldn't load %s: %s\n", filename, SDL_GetError());
        return false;
    }

	if( SDL_SetColorKey(m_picture, SDL_SRCCOLORKEY | SDL_RLEACCEL, transparentcolor) < 0){
		g_freemain->msg(2, "Couldn't set ColorKey + RLE for %s: %s", filename, SDL_GetError());
		return false;
	}

	SDL_Surface *temp;
	if( (temp = SDL_DisplayFormat(m_picture)) == NULL){
		g_freemain->msg(2, "couldn't convert %s to display format: %s", filename, SDL_GetError());
		return false;
	}
	SDL_FreeSurface(m_picture);
	m_picture = temp;


	m_bltrect.w = picwidth;
	m_bltrect.h = picheight;

	m_picrect.w = picwidth;
	m_picrect.h = picheight;

	m_lastpicswitch = SDL_GetTicks();
	m_delay = delay;

	int i, j, k, piccount_hor, piccount_vert;

	piccount_hor	= ((m_picture->w)-1) / (picwidth+1);
	piccount_vert	= ((m_picture->h)-1) / (picheight+1);

	m_anzbilder	= piccount_hor*piccount_vert;
	m_aktbild = 0;

	k=0;//bilder index

	m_xoffsets = new int[m_anzbilder];
	m_yoffsets = new int[m_anzbilder];

	g_freemain->msg(2, "done loading, calculating offsets");

	if(m_xoffsets == NULL || m_yoffsets == NULL){
		g_freemain->msg(2, "couldn't get memory for x/y offsets for %s!\n", filename);
		return false;
	}

	//offsets fr die sub bilder festlegen (zwischen den bilder befindet sich 1px rand, um alle bilder ist auch 1px rand
	for(i=0; i<piccount_vert; i++){
		for(j=0; j<piccount_hor; j++){
			m_yoffsets[k] = 1 + (1+picheight)*i;
			m_xoffsets[k] = 1 + (1+picwidth)*j;

			k++;
		}
	}


	m_picrect.x = m_xoffsets[0];
	m_picrect.y = m_yoffsets[0];

	g_freemain->msg(2, "DONE - load AnimSprite");
	return true;
}


freeAnimSprite::~freeAnimSprite(){
	if(m_xoffsets != NULL)
		delete [] m_xoffsets;
	if(m_yoffsets != NULL)
		delete [] m_yoffsets;
}

freeAnimSprite::freeAnimSprite(){
	m_picture		= NULL;
	m_bltrect.x		= 0;
	m_bltrect.y		= 0;
	m_bltrect.w		= 0;
	m_bltrect.h		= 0;
	m_picrect.x		= 0;
	m_picrect.y		= 0;;
	m_picrect.w		= 0;;
	m_picrect.h		= 0;;

	m_xoffsets		= NULL;
	m_yoffsets		= NULL;

	m_lastpicswitch	= 0;
	m_delay			= 0;
	m_aktbild		= 0;
	m_anzbilder		= 0;
}


bool freeAnimSprite::draw(int x, int y){
	Uint32 tickcount;

	m_bltrect.x = x;
	m_bltrect.y = y;

	if(SDL_BlitSurface(m_picture, &m_picrect, g_freemain->getScreen(), &m_bltrect) < 0){
		fprintf(stderr, "BlitSurface error: %s\n", SDL_GetError());
		return false;
	}



	if( ((tickcount=SDL_GetTicks()) - m_lastpicswitch) >= m_delay){
		m_aktbild += (tickcount-m_lastpicswitch)/m_delay;

		if(m_aktbild >= m_anzbilder){
			m_aktbild = m_aktbild % m_anzbilder;
		}

		m_lastpicswitch=tickcount;

		m_picrect.x = m_xoffsets[m_aktbild];
		m_picrect.y = m_yoffsets[m_aktbild];
	}

	return true;
}


bool freeAnimSprite::draw(int x, int y, int PicNum){
	m_bltrect.x = x;
	m_bltrect.y = y;

	SDL_Rect r;
	r.x = m_xoffsets[PicNum%m_anzbilder];
	r.y = m_xoffsets[PicNum%m_anzbilder];
	r.w = m_picrect.w;
	r.h = m_picrect.h;

	if(SDL_BlitSurface(m_picture, &r, g_freemain->getScreen(), &m_bltrect) < 0){
		fprintf(stderr, "BlitSurface error: %s\n", SDL_GetError());
		return false;
	}

	return true;
}







freeFont::freeFont(){
	memset(&m_font,0,sizeof(SFont_FontInfo));
}


freeFont::~freeFont(){
	SDL_FreeSurface(m_font.Surface);
}


bool freeFont::init(const char *filename){
// Load the BMP file into a surface
	g_freemain->msg(1, "loading Font %s", filename);
    m_font.Surface = IMG_Load(filename);

    if (m_font.Surface == NULL) {
        g_freemain->msg(2, "Couldn't load %s: %s\n", filename, SDL_GetError());
        return false;
    }

	g_freemain->msg(2, "InitFont2 %s\n", filename);
	InitFont2(&m_font);

	g_freemain->msg(2, "Done!\n");
	return true;
}


bool freeFont::draw(int x, int y, char *s){
	PutString2(g_freemain->getScreen(), &m_font, x, y, s);
	return true;
}


bool freeFont::drawf(int x, int y, char *s, ...){
	char buffer[256];

	va_list zeiger;
	va_start(zeiger, s);
	vsprintf(buffer, s, zeiger);
	va_end(zeiger);

	return draw(x,y,buffer);
}


int	 freeFont::GetCharWidth(char c){
	int ofs=0;
	if(c==' ') {
		return this->m_font.CharPos[2]-this->m_font.CharPos[1];
	}
	else {
		ofs=((unsigned char)c-33)*2+1;
		return this->m_font.CharPos[ofs+1] - this->m_font.CharPos[ofs];
	}
	return -1;
}


int freeFont::GetStringWidth(char *s){
	int				ofs;					//offset in sfont
	int				ges		= 0;			//gesamtlnge
	SFont_FontInfo  &font	= this->m_font;	//beschleunigt die angelegenheit ein bisschen, da nicht dauernd neu dereferenziert werden muss

	while(*s){
		ofs = ((unsigned char)(*s)-33)*2+1;				//offset ausrechnen
		ges += font.CharPos[ofs+1] - font.CharPos[ofs];	//gesamtlnge += charwidth
		s++;											//nchstes zeichen
	}

	return ges;
}

//last changes: tijot 0.89.3 - no_skill

