/*
  projektname:		tijot

  beschreibung:		ein tetris clone
  authoren:			jobe, z-man, no_skill
  website:			www.reddesert.de

  beginn:			August 2003

  technologien:		sdl / sdl_image / sfont / fmod
                    alles zusammen in der tfree engine - written by no_skill and JoBe

  last changes:		tijot 0.89.6.a  @  12.01.2004 - z-man
*/

//Defines
#define RD_HIGHSCOREFILE	"highscore.txt"
#define RD_HIGHSCORECODE	"dssQWasdFRgasFgokwergoskvsXgksdgGskvm"

#define RD_VERSION			"0.89.6"

#define RD_STARTGAME		1
#define RD_STARTCHALLENGE	2
#define RD_OPTIONS			3
#define RD_INFO				4
#define RD_QUIT				5
#define RD_HIGHSCORE		6

#define RD_SIGNDELAY		1000

#include "tfree/tfree.h"		//free engine by no_skill - abge?ndert auf tetris kompatibilit?t
//^^ inkl: sdl.h #include <sdl_image.h>
//^^ inkl: clog.h
//^^ inkl: #include <fstream.h> #include <time.h> #include <stdarg.h> #include <stdio.h>
#include "zman/RDTetris.h"		//Tetris klassen by z-man
#include "zman/RDHighscore.h"	//Highscore klassen by z-man
#include "zman/RDMen.h"		//Men? klassen by z-man



//Variables
freeMain	Engine;
freeFont	Text;
freeFont	TextM;
CSound		snd_background;



SDL_Event event;

//k?nnen durch command line arguments ver?ndert werden
bool debug		= false; //Global debug
bool fullscreen = true;  //fullscreen mode
bool bgsound	= true;	 //Background music


//Function predefines
bool	CallGame(void);
bool	CallChallenge(void);
bool	CallOptions(void);
bool	CallHighscore(void);
bool	CallInfo(void);
int		CallMenue(void);
bool	CallGetName(char chMessage[],char *chGet,unsigned int uiMax);


bool	DrawGimmicks(int x,int y,int iLevel,RDTetris *pTetris,freeFont *pText);

bool	RefreshHighscore(char chName[],long int liPoints,long int liLevel,long int liTime);
bool	CheckHighscore(long int liPoints,long int liLevel,long int liTime);



//Main
int main(int argc, char **argv)
{
bool done		= false;
	//Check Commandline Attributes
	int i;

	//falls mehr als die pfadangabe da is, wird das rausgeparsed
	//falls nur der pfad da is, is argc = 1 und er geht gar nicht in die for schleife
	for(i = 1; i < argc; i++){
		if(strcmp(argv[i], "--help") == 0){
			fprintf(stdout, "usage:\n -w: windowed\n -debug: debug mode\n --help: this text\n");
			return 1;
		}
		else if(strcmp(argv[i], "-w") == 0){
			fullscreen = false;
		}
		else if(strcmp(argv[i], "-debug") == 0){
			debug = true;
		}
	}
	


	//Init basic engine
	if(!Engine.init(640,480,fullscreen,"tijotlog.txt")) {
		return 1;
	}
	Engine.msg(1,"Engine init was succesfull!");
	//Write basic info
	if(debug) {
		Engine.msg(1,"Debug mode is activated!");
	}

	
	if(!Text.init("data/grafik/font.bmp")) {
		Engine.msg(2,"Init Text engine failed!");
		return false;
	}
	if(!TextM.init("data/grafik/font_orange.bmp")) {
		Engine.msg(2,"Init Text marked engine failed!");
		return false;
	}
	Engine.msg(2,"Text engine ready!");

	// Loading Sounds
	snd_background.Load("data/sound/background.mp3");

	// Starting background-sound (true bedeutet hier unendlich oft abspielen)
	snd_background.Play(true);
	
	//Hide Mouse
	SDL_ShowCursor(false);




	//Run outer Mainloop
	Engine.msg(1,"Starting outer Mainloop!");
	while(!done) {
		//Call Menue
		switch(CallMenue()) {
			case RD_STARTGAME:
				//Start Game
				Engine.msg(1,"StartGame");
				if(!CallGame()) {
					Engine.msg(1,"Error in Game");
					done=true;
				}
			break;

			case RD_STARTCHALLENGE:
				//Start Challenge
				Engine.msg(1,"StartChallenge");
				if(!CallChallenge()) {
					Engine.msg(1,"Error in Challenge");
					done=true;
				}
			break;

			case RD_HIGHSCORE:
				//Highscore
				Engine.msg(1,"StartHighscore");
				if(!CallHighscore()) {
					Engine.msg(1,"Error in Highscore");
					done=true;
				}
			break;

			case RD_OPTIONS:
				//Options
				Engine.msg(1,"StartOptions");
				if(!CallOptions()) {
					Engine.msg(1,"Error in Options");
					done=true;
				}
			break;

			case RD_INFO:
				//Info
				Engine.msg(1,"StartInfo");
				if(!CallInfo()) {
					Engine.msg(1,"Error in Info");
					done=true;
				}
			break;

			case RD_QUIT:
				//Quit
				Engine.msg(1,"User chose Exit!");
				done=true;
			break;

			default:
				Engine.msg(1,"Error in Menue");
				done=true;
			break;
		}
	}
	//Unhide Mouse
	SDL_ShowCursor(true);
	Engine.msg(1,"Outer Mainloop finished, Goodbye!");
	return 0;
} 


//////////////////
//Game functions//
//////////////////


bool	CallGame(void){
	char		chName[100];
	CSound		snd_stonedown;
	CSound		snd_crackline;
	CSound		snd_loose;
	CSound		snd_highscore;
	int			iLevel=0;
	int			Action=0;
	int			iTmp=0;
	RDTetris	Tetris;
	unsigned int Timer;

	//Additional bonus sign parameters
	unsigned int uiSignTimer=0;
	int			iLPoints=0;
	int			iLLines=0;

	unsigned char Minutes=0;
	unsigned char Seconds=0;
	unsigned int Clock=0;
	int			iSpeed=0;

	bool		done	=false;
	bool		Paused	=false;

	//Init everything

	Engine.msg(1,"Game called...");
	if(!Tetris.Init(400)) {
		Engine.msg(2,"Init Tetris engine failed!");
		return false;
	}
	Engine.msg(2,"Tetris engine ready!");

	if(!snd_stonedown.Load("data/sound/stonedown.wav") || !snd_crackline.Load("data/sound/crackline.wav") || !snd_loose.Load("data/sound/loose.wav")
		|| !snd_highscore.Load("data/sound/highscore.wav")) {
		Engine.msg(2,"Load Tetris Sound failed!");
		return false;
	}
	Tetris.SetSound(RDTM_STONEDOWN,&snd_stonedown);
	Tetris.SetSound(RDTM_CRACKLINE,&snd_crackline);
	Engine.msg(2,"Tetris Sound loaded!");

	if(!RDTLoadGraphics("data/grafik/graphic.txt",&Tetris)) {
		Engine.msg(2,"Load Tetris Graphics failed!");
		return false;
	}
	Engine.msg(2,"Tetris Graphics loaded!");

	//Enter mainloop
	Engine.msg(2,"Enter inner Mainloop");
	Clock=uiSignTimer=Timer=SDL_GetTicks();
	

	while(!done) {		//Mainloop
		//Clear Screen
		if(SDL_FillRect(Engine.getScreen(),NULL,0x000000)==-1) {
			Engine.msg(2,"Fill Screen failed!");
			break;
		}
		//Update MSG
		while(SDL_PollEvent(&event)){
			if (event.type == SDL_QUIT){
				done = true;
				Engine.msg(1, "user killed window! - exit");
			}
		}

		//Check Standart Input
		Action=RDTA_NOTHING;	//Reset old Input
		if(Engine.pressed(SDLK_UP)) { //Up
			Action |=RDTA_ROTATE;
		}
		if(Engine.pressed(SDLK_RIGHT)) { //Right
			Action |=RDTA_RIGHT;
		}
		if(Engine.pressed(SDLK_DOWN)) { //Down
			Action |=RDTA_SPEEDUP;
		}
		if(Engine.pressed(SDLK_LEFT)) { //Left
			Action |=RDTA_LEFT;
		}
		if(Engine.pressed(SDLK_ESCAPE)) { //Escape
			done=true;
		}

		//Entprellte Tasten
		if(SDL_GetTicks()-Timer>200) {
			Timer=SDL_GetTicks();
			if(Engine.pressed(SDLK_p)) { //P-p-pause*g*
				if(Paused) {
					Paused=false;
				}
				else {
					Paused=true;
				}
			}
			else if(Engine.pressed(SDLK_m)) {	//Sound an/aus*g*
				if(bgsound) {
					snd_background.Stop();
					bgsound=false;
				}
				else {
					snd_background.Play(true);
					bgsound=true;
				}
			}


		}
		//Run Tetris
		if(!Paused) {
			if(!Tetris.Run(Action)) {
				if(Tetris.Lost()) {
					Engine.msg(2,"Game Over");
					done=true;
				}
				else {
					Engine.msg(2,"Run Tetris failed!");
					break;
				}
			}
			//Sign add
			if(Tetris.GetLastPoints()!=0) {
				iLPoints=Tetris.GetLastPoints();
				iLLines	=Tetris.GetLastLines();
				uiSignTimer=SDL_GetTicks();
			}
			//Refresh Clock
			if(SDL_GetTicks()-Clock>1000) {
				SDL_GetTicks();
				Seconds+=((SDL_GetTicks()-Clock)/1000);
				if(Seconds>59) {
					Seconds-=60;
					if(Minutes==255) {
						Engine.msg(2,"Clock out of Bound!"); 
						done=true;
					}
					Minutes++;
				}
				Clock=SDL_GetTicks();
			}
		}
		//Level Controll
		if(Tetris.GetLines()>=10*iLevel) {
			iLevel++;
			iSpeed=Tetris.GetSpeed();
			iSpeed+=iLevel*50;
			Tetris.SetSpeed(iSpeed);
			Engine.msg(2,"Start Level %i",iLevel);
			Engine.msg(2,"Calculated Speed: %i",iSpeed);
			Timer=SDL_GetTicks();
			while(SDL_GetTicks()-Timer<2000) {
				Clock=SDL_GetTicks();
				//Clear Screen
				if(SDL_FillRect(Engine.getScreen(),NULL,0x000000)==-1) {
					Engine.msg(2,"Fill Screen failed!");
					break;
				}
				//Update MSG
				while(SDL_PollEvent(&event)){
					if (event.type == SDL_QUIT){
						done = true;
						Engine.msg(1, "user killed window! - exit");
					}
				}
				//Draw field
				if(!Tetris.Draw(200,10)) {
					Engine.msg(2,"Draw Tetris failed!");
					break;
				}
				//Draw Level
				if(!TextM.drawf(270,200,"Level %i",iLevel)) {
					Engine.msg(2,"Draw Level failed!");
					break;
				}
				//Draw Clock
				if(!Text.drawf(20,10,"%i:%i",Minutes,Seconds)) {
					Engine.msg(2,"Draw Clock failed!");
					break;
				}
				//Draw gimmicks
				if(!DrawGimmicks(20,100,iLevel,&Tetris,&Text)) {
					Engine.msg(2,"Draw Gimmicks failed!");
					break;
				}
				//Flip Screen
				if(!Engine.flip()) {
					Engine.msg(2,"Flip failed!");
					break;
				}
			}
		}
		//Draw field
		if(!Tetris.Draw(200,10)) {
			Engine.msg(2,"Draw Tetris failed!");
			break;
		}
		if(Paused) {
			//Don't run the clock
			Clock=SDL_GetTicks();
			//Draw paused label
			if(!TextM.draw(270,200,"Paused")) {
				Engine.msg(2,"Draw Paused failed!");
				break;
			}
		}
		else {
			if(iLPoints!=0) {
				if(SDL_GetTicks()-uiSignTimer<RD_SIGNDELAY) {
					if(!TextM.drawf(270,200,"%i x %i = %i",iLLines,iLLines*10,iLPoints)) {
						Engine.msg(2,"Draw sign points failed!");
						break;
					}
				}
			}

		}
		//Draw Clock
		if(!Text.drawf(20,10,"%i:%i",Minutes,Seconds)) {
			Engine.msg(2,"Draw Clock failed!");
		}
		//Draw gimmicks
		if(!DrawGimmicks(20,100,iLevel,&Tetris,&Text)) {
			Engine.msg(2,"Draw Gimmicks failed!");
			break;
		}
		//Flip Screen
		if(!Engine.flip()) {
			Engine.msg(2,"Flip failed!");
			break;
		}

	}
	//Draw Game Over
		//Clear Screen
	snd_loose.Play();

	if(done) {
		Timer=SDL_GetTicks();
		while((SDL_GetTicks()-Timer)<2000) {		//Display 2 seconds a Game Over screen
			if(SDL_FillRect(Engine.getScreen(),NULL,0x000000)==-1) {
				Engine.msg(2,"Fill Screen failed!");
				done=false;
				break;
			}	
			//Update MSG
			while(SDL_PollEvent(&event)){
				if (event.type == SDL_QUIT){
					done = true;
					Engine.msg(1, "user killed window! - exit");
				}
			}
			//Draw field
			if(!Tetris.Draw(200,10)) {
				Engine.msg(2,"Draw Tetris failed!");
				done=false;
				break;
			}
			//Draw Clock
			if(!Text.drawf(20,10,"%i:%i",Minutes,Seconds)) {
				Engine.msg(2,"Draw Clock failed!");
				done=false;
				break;
			}
			//Draw gimmicks
			if(!DrawGimmicks(20,100,iLevel,&Tetris,&Text)) {
				Engine.msg(2,"Draw Gimmicks failed!");
				done=false;
				break;
			}
			//Draw Game Over
			if(!TextM.drawf(270,200,"Game Over")) {
				Engine.msg(2,"Draw Game Over failed!");
				done=false;
				break;
			}
			//Flip Screen
			if(!Engine.flip()) {
				Engine.msg(2,"Flip failed!");
				done=false;
				break;
			}
		}
		//Get Name


		//DEBUG
		if(CheckHighscore(Tetris.GetPoints(),iLevel,Minutes*60+Seconds)) {
			//Get Name
			if(SDL_FillRect(Engine.getScreen(),NULL,0x000000)==-1) {
				Engine.msg(2,"Fill Screen failed!");
				return false;
			}	
			if(!CallGetName("Enter your name",chName,100)) {
				Engine.msg(2,"Get name failed!");
				return false;
			}
			//SFont_Input(Engine.getScreen(),200,240,500,chName);
			if(RefreshHighscore(chName,Tetris.GetPoints(),iLevel,Minutes*60+Seconds)) {
				snd_highscore.Play();
				if(!CallHighscore()){
					return false;
				}
			}
		}
		//DEBUG ENDE

	}
	//Free everything
	Engine.msg(2,"Release Point reached!");
	RDTUnloadGraphics(&Tetris);
	Engine.msg(2,"Unloaded Tetris Graphics");
	snd_stonedown.Stop();
	snd_crackline.Stop();
	snd_highscore.Stop();
	snd_loose.Stop();
	Engine.msg(2,"Unloaded Tetris Sounds");
	Tetris.Reset();

	Engine.msg(2,"Tetris Reset Done");
	if(!done) return false;
	return true;
}//bool	CallGame(void)


bool	CallChallenge(void){
	CSound		snd_stonedown;
	CSound		snd_crackline;
	CSound		snd_highscore;
	CSound		snd_loose;
	bool		done=false;
	bool		Paused=false;
	int			i=0;
	int			ActionLeft;
	int			ActionRight;

	unsigned int uiSignTimerLeft=0;
	unsigned int uiSignTimerRight=0;
	int			iLPointsLeft=0;
	int			iLPointsRight=0;
	int			iLLinesLeft=0;
	int			iLLinesRight=0;

	unsigned int Timer=0;

	unsigned int Clock=0;
	unsigned char Minutes=0;
	unsigned char Seconds=0;
	
	RDTetris	PlayerLeft;
	RDTetris	PlayerRight;

	//Load everything
	Engine.msg(2,"Challenge called...");
	
	if(!PlayerLeft.Init(400)) {
		Engine.msg(2,"Init Left Player failed!");
		return false;;
	}
	if(!PlayerRight.Init(400)) {
		Engine.msg(2,"Init Right Player failed!");
		return false;
	}

	if(!snd_stonedown.Load("data/sound/stonedown.wav") || !snd_crackline.Load("data/sound/crackline.wav") || !snd_loose.Load("data/sound/loose.wav")
		|| !snd_highscore.Load("data/sound/highscore.wav")) {
		Engine.msg(2,"Load Tetris Sound failed!");
		return false;
	}
	PlayerRight.SetSound(RDTM_STONEDOWN,&snd_stonedown);
	PlayerRight.SetSound(RDTM_CRACKLINE,&snd_crackline);
	PlayerLeft.SetSound(RDTM_STONEDOWN,&snd_stonedown);
	PlayerLeft.SetSound(RDTM_CRACKLINE,&snd_crackline);

	Engine.msg(2,"Tetris Sound loaded!");

	Engine.msg(2,"Loading Graphics for Left player...");
	if(!RDTLoadGraphics("data/grafik/graphic.txt",&PlayerLeft)) {
		Engine.msg(2,"FAILED");
		return false;
	}
	Engine.msg(2,"DONE");
	Engine.msg(2,"Mirror Graphics to Right player...");
	for(i=-1;i<=RDTG_SPRITES;i++) {
		PlayerRight.SetGraphic(i,PlayerLeft.GetGraphic(i));
	}
	Engine.msg(2,"DONE");

	Engine.msg(2,"Enter inner mainloop...");
	Timer=Clock=uiSignTimerLeft=uiSignTimerRight=SDL_GetTicks();
	while(!done) {
		if(SDL_FillRect(Engine.getScreen(),NULL,0x000000)==-1) {
			Engine.msg(2,"Fill Screen failed!");
			break;
		}
		while(SDL_PollEvent(&event)){
			if (event.type == SDL_QUIT){
				done = true;
				Engine.msg(1, "user killed window! - exit");
			}
		}
		//Check Standart Input
		ActionLeft	=RDTA_NOTHING;	//Reset old Left Input
		ActionRight	=RDTA_NOTHING;	//Reset old Right Input
		//Check Right Player input
		if(Engine.pressed(SDLK_UP)) { //Up
			ActionRight |=RDTA_ROTATE;
		}
		if(Engine.pressed(SDLK_RIGHT)) { //Right
			ActionRight |=RDTA_RIGHT;
		}
		if(Engine.pressed(SDLK_DOWN)) { //Down
			ActionRight |=RDTA_SPEEDUP;
		}
		if(Engine.pressed(SDLK_LEFT)) { //Left
			ActionRight |=RDTA_LEFT;
		}
		//Check Left Player input
		if(Engine.pressed(SDLK_w)) { //Up
			ActionLeft |=RDTA_ROTATE;
		}
		if(Engine.pressed(SDLK_d)) { //Right
			ActionLeft |=RDTA_RIGHT;
		}
		if(Engine.pressed(SDLK_s)) { //Down
			ActionLeft |=RDTA_SPEEDUP;
		}
		if(Engine.pressed(SDLK_a)) { //Left
			ActionLeft |=RDTA_LEFT;
		}
		//Check exit
		if(Engine.pressed(SDLK_ESCAPE)) { //Escape
			Engine.msg(2,"User exits!");
			done=true;
		}
		//Entprellte Tasten
		if(SDL_GetTicks()-Timer>200) {
			Timer=SDL_GetTicks();
			if(Engine.pressed(SDLK_p)) { //P-p-paused*g*
				if(Paused) {
					Paused=false;
				}
				else {
					Paused=true;
				}
			}
			else if(Engine.pressed(SDLK_m)) {	//Sound an/aus*g*
				if(bgsound) {
					snd_background.Stop();
					bgsound=false;
				}
				else {
					snd_background.Play(true);
					bgsound=true;
				}
			}
		}	
		//Run Tetris
		if(!Paused) {
			if(!PlayerLeft.Run(ActionLeft)) {
				if(PlayerLeft.Lost()) {
					Engine.msg(2,"Left Player lost!");
					done=true;
				}
				else {
					Engine.msg(2,"Run PlayerLeft failed!");
					break;
				}
			}
			//Sign add
			if(PlayerLeft.GetLastPoints()!=0) {
				iLPointsLeft=PlayerLeft.GetLastPoints();
				iLLinesLeft	=PlayerLeft.GetLastLines();
				uiSignTimerLeft=SDL_GetTicks();
			}

			if(!PlayerRight.Run(ActionRight)) {
				if(PlayerRight.Lost()) {
					Engine.msg(2,"Right Player lost!");
					done=true;
				}
				else {
					Engine.msg(2,"Run PlayerRight failed!");
					break;
				}
			}

			//Sign add
			if(PlayerRight.GetLastPoints()!=0) {
				iLPointsRight	=PlayerRight.GetLastPoints();
				iLLinesRight	=PlayerRight.GetLastLines();
				uiSignTimerRight=SDL_GetTicks();
			}
			//Refresh Clock
			if(SDL_GetTicks()-Clock>1000) {
				SDL_GetTicks();
				Seconds+=((SDL_GetTicks()-Clock)/1000);
				if(Seconds>59) {
					Seconds-=60;
					if(Minutes==255) {
						Engine.msg(2,"Clock out of Bound!"); 
						done=true;
					}
					Minutes++;
				}
				Clock=SDL_GetTicks();
			}


		}
		//Use Balance system
		//Work out speed up on points
		if(PlayerLeft.GetPoints()>PlayerRight.GetPoints()) {
			PlayerRight.SetSpeed(400+5*(PlayerLeft.GetPoints()-PlayerRight.GetPoints()));
		}
		else if(PlayerRight.GetPoints()<PlayerLeft.GetPoints()) {
			PlayerLeft.SetSpeed(400+5*(PlayerRight.GetPoints()-PlayerLeft.GetPoints()));
		}
		//Draw field
		if(!PlayerLeft.Draw(10,10)) {
			Engine.msg(2,"Draw PlayerLeft failed!");
			break;
		}
		if(!PlayerRight.Draw(380,10)) {
			Engine.msg(2,"Draw PlayerRight failed!");
			break;
		}
		if(Paused) {
			//Don't run the clock
			Clock=SDL_GetTicks();
			//Draw paused label
			if(!TextM.draw(60,200,"Paused")) {
				Engine.msg(2,"Draw Paused failed!");
				break;
			}
			if(!TextM.draw(460,200,"Paused")) {
				Engine.msg(2,"Draw Paused failed!");
				break;
			}
		}
		else {
			if(iLPointsLeft!=0) {
				if(SDL_GetTicks()-uiSignTimerLeft<RD_SIGNDELAY) {
					if(!TextM.drawf(60,200,"%i x %i = %i",iLLinesLeft,iLLinesLeft*10,iLPointsLeft)) {
						Engine.msg(2,"Draw sign points left failed!");
						break;
					}
				}
			}
			if(iLPointsRight!=0) {
				if(SDL_GetTicks()-uiSignTimerRight<RD_SIGNDELAY) {
					if(!TextM.drawf(460,200,"%i x %i = %i",iLLinesRight,iLLinesRight*10,iLPointsRight)) {
						Engine.msg(2,"Draw sign points right failed!");
						break;
					}
				}
			}


		}
		//Draw Clock
		if(!Text.drawf(290,10,"%i:%i",Minutes,Seconds)) {
			Engine.msg(2,"Draw Clock failed!");
		}
		//Draw gimmicks
		if(!DrawGimmicks(255,50,-1,&PlayerLeft,&Text)) {
			Engine.msg(2,"Draw Gimmicks left failed!");
			break;
		}
		if(!DrawGimmicks(255,220,-1,&PlayerRight,&Text)) {
			Engine.msg(2,"Draw Gimmicks Right failed!");
			break;
		}
		//Flip
		if(!Engine.flip()) {
			Engine.msg(2,"Flip screen failed!");
			break;
		}
	}
	if(PlayerLeft.GetPoints()-PlayerRight.GetPoints()!=0) {
		snd_highscore.Play();
	}
	else {
		snd_loose.Play();
	}

	Engine.msg(2,"Tetris Sound loaded!");
	//Say which player won*g*
	//Wait 0,3 secs
	Timer=SDL_GetTicks();
	while(SDL_GetTicks()-Timer<300) {};

	if(done) {
		Timer=SDL_GetTicks();
		done=false;
		while(!done) {		
			if(SDL_FillRect(Engine.getScreen(),NULL,0x000000)==-1) {
				Engine.msg(2,"Fill Screen failed!");
				done=false;
				break;
			}	
			//Update MSG
			while(SDL_PollEvent(&event)){
				if (event.type == SDL_QUIT){
					done = true;
					Engine.msg(1, "user killed window! - exit");
				}
			}
			if(Engine.pressed(SDLK_RETURN) || Engine.pressed(SDLK_SPACE) || Engine.pressed(SDLK_ESCAPE)) {
				Engine.msg(2,"User exits!");
				done=true;
			}
			//Draw field
			if(!PlayerLeft.Draw(10,10)) {
				Engine.msg(2,"Draw PlayerLeft failed!");
				break;
			}
			if(!PlayerRight.Draw(380,10)) {
				Engine.msg(2,"Draw PlayerRight failed!");
				break;
			}
			//Draw Clock
			if(!Text.drawf(290,10,"%i:%i",Minutes,Seconds)) {
				Engine.msg(2,"Draw Clock failed!");
			}
			//Draw gimmicks
			if(!DrawGimmicks(255,50,-1,&PlayerLeft,&Text)) {
				Engine.msg(2,"Draw Gimmicks left failed!");
				break;
			}
			if(!DrawGimmicks(255,220,-1,&PlayerRight,&Text)) {
				Engine.msg(2,"Draw Gimmicks Right failed!");
				break;
			}
			//Draw Game Over
			if(PlayerLeft.GetPoints()-PlayerRight.GetPoints()>0) {
				//Right Player lost
				if(!TextM.drawf(60,200,"You Won")) {
					Engine.msg(2,"Draw Game Over failed!");
					break;
				}
				if(!TextM.drawf(460,200,"You Lost")) {
					Engine.msg(2,"Draw Game Over failed!");
					break;
				}
			}
			else if(PlayerLeft.GetPoints()-PlayerRight.GetPoints()<0) {
				//Left Player lost
				if(!TextM.drawf(60,200,"You Lost")) {
					Engine.msg(2,"Draw Game Over failed!");
					break;
				}
				if(!TextM.drawf(460,200,"You Won")) {
					Engine.msg(2,"Draw Game Over failed!");
					break;
				}
			}
			else {
				//Patt
				//Look up who finished the game
				if(PlayerLeft.Lost()) {
					//Left Player lost
					if(!TextM.drawf(60,200,"You Lost")) {
						Engine.msg(2,"Draw Game Over failed!");
						break;
					}
					if(!TextM.drawf(460,200,"You Won")) {
						Engine.msg(2,"Draw Game Over failed!");
						break;
					}
				}
				else if(PlayerRight.Lost()) {
					//Right Player lost
					if(!TextM.drawf(60,200,"You Won")) {
						Engine.msg(2,"Draw Game Over failed!");
						break;
					}
					if(!TextM.drawf(460,200,"You Lost")) {
						Engine.msg(2,"Draw Game Over failed!");
						break;
					}
				}
				else {
					if(!TextM.drawf(60,200,"Patt")) {
						Engine.msg(2,"Draw Game Over failed!");
						break;
					}
					if(!TextM.drawf(460,200,"Patt")) {
						Engine.msg(2,"Draw Game Over failed!");
						break;
					}
				}
			}
			//Flip Screen
			if(!Engine.flip()) {
				Engine.msg(2,"Flip failed!");
				done=false;
				break;
			}
		}
		//Other stuff
	}
	//Release everything
	Engine.msg(2,"Release Point reached!");
	RDTUnloadGraphics(&PlayerLeft);
	snd_stonedown.Stop();
	snd_crackline.Stop();
	snd_highscore.Stop();
	snd_loose.Stop();
	PlayerLeft.Reset();
	PlayerRight.Reset();

	return true;
}//bool	CallChallenge(void)

bool	CallOptions(void){
	
	bool			done=false;
	freeSprite		SndOn;
	freeSprite		SndOff;
	unsigned int	uiTimer=0;
	RDMenKeyboard		kb;
	RDMenMouse		ms;
	RDMenButton		Back;
	RDMenLabel		lb;

	int				x,y,s,k;



	//Load everything
	Engine.msg(2,"Options called...");
	if(!kb.Init()) return false;
	if(!ms.Init("data/grafik/mouse.bmp")) return false;
	if(!Back.Init(RDMenCreateRect(10,410,90,40))) return false;
	if(!Back.ActGrfStandart("data/grafik/back.jpg")) return false;
	Back.ActMouse(&x,&y,&s);

	if(!lb.Init(RDMenCreateRect(10,10,0,0))) return false;
	if(!lb.ActGrfStandart("data/grafik/menu_top.jpg")) return false;

	if(!SndOn.init("data/grafik/sndon.jpg")) return false;
	if(!SndOff.init("data/grafik/sndoff.jpg")) return false;



	Engine.msg(2,"Data loaded!");
	while(!done) {
		if(SDL_FillRect(Engine.getScreen(),NULL,0x000000)==-1) {
			Engine.msg(2,"Fill Screen failed!");
			break;
		}
		//Run message services
		if(!kb.Run()) {
			Engine.msg(2,"User closed window!");
			break;
		}
		if(!ms.Run()) {
			Engine.msg(2,"Update Mouse failed!");
			break;
		}

		kb.GetState(&k);
		ms.GetState(&x,&y,&s);
		//Run other objects
		if(!Back.Run()) {
			Engine.msg(2,"Run back Button failed!");
			break;
		}

		//Check events
		if(Back.GetState()==RDMEN_PRESSED) {
			Engine.msg(2,"User pressed back button!");
			done=true;
		}

		if(k!=0) {
			Engine.msg(2,"User exits!");
			done=true;
		}

		//Special bounding for sound on/off switch button
		if(s & RDMEN_KLICKLEFT && x>200 && x<250 && y>200 && y<250) {
			if(SDL_GetTicks()-uiTimer>300) {
				uiTimer=SDL_GetTicks();
				if(bgsound) {
					snd_background.Stop();
					bgsound=false;
				}
				else {
					snd_background.Play(true);
					bgsound=true;
				}
			}
		}

		//Draw sound symbols
		if(bgsound) {
			SndOn.draw(200,200);
		}else {
			SndOff.draw(200,200);
		}
		//Draw Label
		if(!lb.Draw()) {
			Engine.msg(2,"Draw Label failed!");
			break;
		}
		//Draw button
		if(!Back.Draw()) {
			Engine.msg(2,"Draw Back button failed!");
			break;
		}
		//Draw Sound info
		if(!Text.draw(200,170,"Sound")) {
			Engine.msg(2,"Draw Sound info failed!");
			break;
		}
		//Draw Version
		if(!Text.drawf(540,450,"V%s",RD_VERSION)) {
			Engine.msg(2,"Draw Version failed!");
			break;
		}
		//Draw mouse
		if(!ms.Draw()) {
			Engine.msg(2,"Draw Mouse failed!");
			break;
		}
		
		//Flip
		if(!Engine.flip()) {
			Engine.msg(2,"Flip screen failed!");
			break;
		}
	}
	Engine.msg(2,"Release point reached!");
	Back.Reset();
	lb.Reset();
	ms.Reset();
	kb.Reset();
	//Release everything


	if(!done) {
		Engine.msg(2,"Return false");
		return false;
	}
	Engine.msg(2,"Return true");
	return true;
}//bool	CallOptions(void)

bool	CallHighscore(void){
	unsigned int uiDelay=0;
	bool		done=false;
	int				i=0;
	RDHighscore	High;
	
	Engine.msg(2,"Highscore called...");
	//Load Highscore file
	Engine.msg(2,"Load file...");
	if(!High.LoadFile(RD_HIGHSCOREFILE,RD_HIGHSCORECODE)) {
		Engine.msg(2,"Open Highscore failed!");
	}
	else {
		Engine.msg(2,"Load done!");
	}
	//Eckdaten checken
	if(High.GetCount()>10) {	//?bersch?ssige eintr?ge entfernen
		High.Cut(10);
	}
	//In mainloop eintreten
	uiDelay=SDL_GetTicks();

	Engine.msg(2,"Enter inner mainloop...");

	while(!done) {
		if(SDL_FillRect(Engine.getScreen(),NULL,0x000000)==-1) {
			Engine.msg(2,"Fill Screen failed!");
			break;
		}
		while(SDL_PollEvent(&event)){
			if (event.type == SDL_QUIT){
				done = true;
				Engine.msg(1, "user killed window! - exit");
			}
		}
		if(Engine.pressed(SDLK_RETURN) || Engine.pressed(SDLK_SPACE) || Engine.pressed(SDLK_ESCAPE)) {
			if(SDL_GetTicks()-uiDelay>500) {
				uiDelay=SDL_GetTicks();
				Engine.msg(2,"User exits!");
				done=true;
			}
		}
		//Draw the stuff
		if(!Text.drawf(10,10,"Name") || !Text.drawf(180,10,"Level") || !Text.drawf(340,10,"Time")
			|| !Text.drawf(540,10,"Points")) {
			Engine.msg(2,"Draw Highscore head failed!");
			break;
		}

		for(i=1;i<=10;i++) {
			if(High.Get(i)!=NULL) {		//Checken ob da was is*g*
				if(!Text.drawf(10,i*30+40,"%s",High.Get(i)->chName)) {
					Engine.msg(2,"Draw Name failed!");
					break;
				}
				if(!Text.drawf(180,i*30+40,"%i",High.Get(i)->liLevel)) {
					Engine.msg(2,"Draw Level failed!");
					break;
				}
				if(High.Get(i)->liTime<=0) {
					Engine.msg(2,"Divide by 0 error!");
					break;
				}
				if(!Text.drawf(340,i*30+40,"%i:%i",High.Get(i)->liTime/60,High.Get(i)->liTime-(High.Get(i)->liTime/60)*60)) {
					Engine.msg(2,"Draw Time failed!");
					break;
				}
				if(!Text.drawf(540,i*30+40,"%i",High.Get(i)->liPoints)) {
					Engine.msg(2,"Draw points failed!");
					break;
				}
			}
			else {			//Wenn nich kommt Mr. nobody auf den platz
				if(!Text.drawf(10,i*30+40,"Nobody")) {
					Engine.msg(2,"Draw Name failed!");
					break;
				}
				if(!Text.drawf(180,i*30+40,"0")) {
					Engine.msg(2,"Draw Level failed!");
					break;
				}
				if(!Text.drawf(340,i*30+40,"0:0")) {
					Engine.msg(2,"Draw Time failed!");
					break;
				}
				if(!Text.drawf(540,i*30+40,"0")) {
					Engine.msg(2,"Draw points failed!");
					break;
				}
			}
		}
		//Draw Version
		if(!Text.drawf(540,450,"V%s",RD_VERSION)) {
			Engine.msg(2,"Draw Version failed!");
			break;
		}
		
		//Flip
		if(!Engine.flip()) {
			Engine.msg(2,"Flip screen failed!");
			break;
		}
	}
	//Release everything
	High.Reset();

	if(!done) return false;

	return true;
}//bool	CallHighscore(void)

int		CallMenue(void){
	int			iRet=-1;
	int			x,y,s,k;
	bool		done=false;
	//Men? specific
	RDMenMouse			ms;
	RDMenKeyboard		kb;
	//Label

	RDMenLabel			Top;
	//Buttons

	RDMenButton			Start;
	RDMenButton			StartC;
	RDMenButton			Options;
	RDMenButton			Info;
	RDMenButton			Exit;

	//Load Menue & Text Engine
	Engine.msg(2,"Menue Called!");
	
	if(SDL_FillRect(Engine.getScreen(),NULL,0x00000)==-1 ||	
		!Text.drawf(250,200,"Loading...") ||
		!Engine.flip()) {
		Engine.msg(2,"Display Load Message failed!");
		return -1;
	}
	
	//Load all menue data
	//Keyboard/Mouse

	if(!kb.Init())													return -1;

	if(!ms.Init("data/grafik/mouse.bmp"))							return -1;

	//Label
	if(!Top.Init(RDMenCreateRect(10,10,0,0)))						return -1;
	if(!Top.ActGrfStandart("data/grafik/menu_top.jpg"))			return -1;
	//Buttons
	
	if(!Start.Init(RDMenCreateRect(80,150,242,50)))					return -1;
	if(!Start.ActGrfStandart("data/grafik/menu_startgame.jpg"))	return -1;
	if(!Start.ActGrfSelected("data/grafik/menu_startgame_a.jpg")) return -1;
	if(!Start.ActMouse(&x,&y,&s))									return -1;
	if(!Start.ActSndSelected("data/sound/stonedown.wav"))			return -1;

	if(!StartC.Init(RDMenCreateRect(80,200,311,58)))				return -1;
	if(!StartC.ActGrfStandart("data/grafik/menu_startchallenge.jpg"))	return -1;
	if(!StartC.ActGrfSelected("data/grafik/menu_startchallenge_a.jpg"))return -1;
	if(!StartC.ActMouse(&x,&y,&s))										return -1;
	if(!StartC.ActSndSelected("data/sound/stonedown.wav"))			return -1;

	if(!Options.Init(RDMenCreateRect(80,258,163,52)))				return -1;
	if(!Options.ActGrfStandart("data/grafik/menu_options.jpg"))	return -1;
	if(!Options.ActGrfSelected("data/grafik/menu_options_a.jpg"))	return -1;
	if(!Options.ActMouse(&x,&y,&s))									return -1;
	if(!Options.ActSndSelected("data/sound/stonedown.wav"))			return -1;

	if(!Info.Init(RDMenCreateRect(80,310,113,42)))					return -1;
	if(!Info.ActGrfStandart("data/grafik/menu_info.jpg"))			return -1;
	if(!Info.ActGrfSelected("data/grafik/menu_info_a.jpg"))		return -1;
	if(!Info.ActMouse(&x,&y,&s))										return -1;
	if(!Info.ActSndSelected("data/sound/stonedown.wav"))			return -1;

	if(!Exit.Init(RDMenCreateRect(80,352,105,51)))					return -1;
	if(!Exit.ActGrfStandart("data/grafik/menu_quit.jpg"))			return -1;
	if(!Exit.ActGrfSelected("data/grafik/menu_quit_a.jpg"))		return -1;
	if(!Exit.ActMouse(&x,&y,&s))									return -1;
	if(!Exit.ActSndSelected("data/sound/stonedown.wav"))			return -1;


	Engine.msg(2,"Menue Ready!");
	while(!done) {
		//Clear Screen
		if(SDL_FillRect(Engine.getScreen(),NULL,0x00000)==-1) {
			Engine.msg(2,"Fill Screen failed!");
			iRet=-1;
			done=true;
		}
		//Run
		if(!kb.Run()) {
			iRet=-1;
			done=true;
		}
		kb.GetState(&k);
		if(!ms.Run()) {
			iRet=-1;
			done=true;
		}
		ms.GetState(&x,&y,&s);
		if(!Start.Run()) {
			iRet=-1;
			done=true;
		}
		if(!StartC.Run()) {
			iRet=-1;
			done=true;
		}
		if(!Options.Run()) {
			iRet=-1;
			done=true;
		}
		if(!Info.Run()) {
			iRet=-1;
			done=true;
		}
		if(!Exit.Run()) {
			iRet=-1;
			done=true;
		}

			
		if(k==SDLK_h) {
			iRet=RD_HIGHSCORE;
			done=true;
		}
		if(Start.GetState()==RDMEN_PRESSED) {
			iRet=RD_STARTGAME;
			done=true;
		}
		if(StartC.GetState()==RDMEN_PRESSED) {
			iRet=RD_STARTCHALLENGE;
			done=true;
		}
		if(Options.GetState()==RDMEN_PRESSED) {
			iRet=RD_OPTIONS;
			done=true;
		}
		if(Info.GetState()==RDMEN_PRESSED) {
			iRet=RD_INFO;
			done=true;
		}
		if(Exit.GetState()==RDMEN_PRESSED) {
			iRet=RD_QUIT;
			done=true;
		}
		//Draw Menue
		if(!Top.Draw()) {
			iRet=-1;
			done=true;
		}
		if(!Start.Draw()) {
			iRet=-1;
			done=true;
		}
		if(!StartC.Draw()) {
			iRet=-1;
			done=true;
		}
		if(!Options.Draw()) {
			iRet=-1;
			done=true;
		}
		if(!Info.Draw()) {
			iRet=-1;
			done=true;
		}
		if(!Exit.Draw()) {
			iRet=-1;
			done=true;
		}
		if(!ms.Draw()) {
			iRet=-1;
			done=true;
		}
		//Draw Version
		if(!Text.drawf(540,450,"V%s",RD_VERSION)) {
			Engine.msg(2,"Draw Version failed!");
			iRet=-1;
			done=true;
		}
		//Flip Buffers
		if(!Engine.flip()) {
			Engine.msg(2,"Flip Buffers failed!");
			iRet=-1;
			done=true;
		}
	}
	Engine.msg(2,"Reset point reached!");
	Top.Reset();
	Start.Reset();
	StartC.Reset();
	Options.Reset();
	Info.Reset();
	Exit.Reset();
	ms.Reset();
	kb.Reset();
	Engine.msg(2,"Reset done!");
	return iRet;

}//int		CallMenue(void)


bool CallInfo(void) {
	bool		done=false;
	freeSprite	Logo;

	//Load everything
	Engine.msg(2,"Info called...");

	
	if(!Logo.init("data/grafik/menu_top.jpg")) {
		Engine.msg(2,"Load Logo failed!");
		return false;
	}
	Engine.msg(2,"Logo loaded!");

	while(!done) {
		if(SDL_FillRect(Engine.getScreen(),NULL,0x000000)==-1) {
			Engine.msg(2,"Fill Screen failed!");
			break;
		}
		while(SDL_PollEvent(&event)){
			if (event.type == SDL_QUIT){
				done = true;
				Engine.msg(1, "user killed window! - exit");
			}
			else if(event.type==SDL_KEYDOWN) {
				Engine.msg(2,"user exits!");
				done=true;
			}
		}
		//Draw the stuff
		if(!Logo.draw(200,20)) {
			Engine.msg(2,"Draw Logo failed!");
			break;
		}
		//Draw Version
		if(!Text.drawf(540,450,"V%s",RD_VERSION)) {
			Engine.msg(2,"Draw Version failed!");
			break;
		}
		//Draw Info
		if(!Text.drawf(10,160,"Tijot is a development of RedDesert")||
			!Text.drawf(10,200,"Features are:") ||
			!Text.drawf(10,240,"Programmed completely portable") ||
			!Text.drawf(10,280,"Crypted Highscore") ||
			!Text.drawf(10,360,"Please visit our Hompage www.RedDesert.de")||
			!Text.drawf(10,420,"Press Enter to Return")) {
			Engine.msg(2,"Draw Info failed!");
			break;
		}
		//Flip
		if(!Engine.flip()) {
			Engine.msg(2,"Flip screen failed!");
			break;
		}
	}
	//Release everything

	if(!done) return false;
	return true;
}//bool CallInfo(void)



bool	DrawGimmicks(int x,int y,int iLevel,RDTetris *pTetris,freeFont *pText) {
	if(pText==NULL|| pTetris==NULL) return false;
	if(iLevel!=-1) {
		if(!pText->drawf(x,y,"Level %i",iLevel)) {
			Engine.msg(2,"Draw Level failed!");
			return false;
		}
	}
	if(!pText->drawf(x,y+40,"Score %i",pTetris->GetPoints())) {
		Engine.msg(2,"Draw Score failed!");
		return false;
	}
	if(!pText->drawf(x,y+80,"Lines %i",pTetris->GetLines())) {
		Engine.msg(2,"Draw Lines failed!");
		return false;
	}
	if(!pText->drawf(x,y+120,"Next")) {
		Engine.msg(2,"Draw Next Label failed!");
		return false;
	}
	if(!pTetris->DrawNext(x+20,y+160)) {
		Engine.msg(2,"Draw Next failed!");
		return false;
	}
	return true;
}

bool	CheckHighscore(long int liPoints,long int liLevel,long int liTime) {
	RDHighscore	High;

	if(!High.LoadFile(RD_HIGHSCOREFILE,RD_HIGHSCORECODE)) {
		Engine.msg(2,"Load Highscore failed!");
	}
	else {
		Engine.msg(2,"Load Highsore done!");
	}

	if(liPoints<=0) return false;	//Keine 0 Punkte zulassen
	//Add element
	if(High.Add(RDHighscoreCreate("testuser",liPoints,liLevel,liTime))>10) { //Wenn nich in die highscore dann vergiss es...
		Engine.msg(2,"User didn't reach highscore!");
		High.Reset();
		return false;
	}
	//Resetten und return true
	High.Reset();
	return true;
};

bool	RefreshHighscore(char chName[],long int liPoints,long int liLevel,long int liTime){
	RDHighscore	High;

	if(!High.LoadFile(RD_HIGHSCOREFILE,RD_HIGHSCORECODE)) {
		Engine.msg(2,"Load Highscore failed!");
	}
	else {
		Engine.msg(2,"Load Highscore done!");
	}
	
	if(liPoints!=0){	//Keine 0 Punkte zulassen
		//Add element
		if(High.Add(RDHighscoreCreate(chName,liPoints,liLevel,liTime))>10) { //Wenn nich in die highscore dann vergiss es...
			Engine.msg(2,"User didn't reach highscore!");
			High.Reset();
			return false;
		}

		//Save List
		if(!High.WriteFile(RD_HIGHSCOREFILE,RD_HIGHSCORECODE)) {
			Engine.msg(2,"Write to Highscore file failed!");
			High.Reset();
			return false;
		}
	}
	//Resetten und return true
	Engine.msg(2,"User reached highscore!");
	High.Reset();
	return true;
};


bool	CallGetName(char chMessage[],char *chGet,unsigned int uiMax){
	int				k;
	RDMenKeyboard	kb;
	RDMenInput		in;
	bool			done=false;

	if(!kb.Init()) return false;
	
	if(!in.Init(RDMenCreateRect(120,180,200,50),20,"data/grafik/cursor.bmp","data/grafik/font.bmp")) {
		Engine.msg(2,"Init Input failed!");
		return false;
	}
	if(!in.ActKeyboard(&k)) return false;

	while(!done) {
		if(SDL_FillRect(Engine.getScreen(),NULL,0x000000)==-1) {
			Engine.msg(2,"Fill Screen failed!");
			break;
		}
		if(!kb.Run()) {
			Engine.msg(2,"User closed windoe!");
			break;
		}
		//Update stats
		kb.GetState(&k);
		//Run
		if(!in.Run()) {
			Engine.msg(2,"Run input failed!");
			break;
		}
		if(in.GetState()==RDMEN_PRESSED) done=true;
		//Draw the stuff
		if(!in.Draw()) {
			Engine.msg(2,"Draw input failed!");
			break;
		}
		if(!Text.drawf(100,100,chMessage)) {
			Engine.msg(2,"Draw Message failed!");
			break;
		}
	
		//Draw Version
		if(!Text.drawf(540,450,"V%s",RD_VERSION)) {
			Engine.msg(2,"Draw Version failed!");
			break;
		}
		//Flip
		if(!Engine.flip()) {
			Engine.msg(2,"Flip screen failed!");
			break;
		}
	}
	//Release everything
	kb.Reset();
	in.GetText(chGet);
	in.Reset();
	if(!done) return false;
	return true;
}


//last changes: tijot 0.89.6.a - z-man

