/****************************************/
/*			RDHighscore.cpp				*/
/*	A simple Highscore wrapper, contains*/
/*  XOR Cryption. Saved values are		*/
/*  Name								*/
/*	Points								*/
/*	Level								*/
/*	Time								*/
/*  This class is developed under the   */
/*  terms of the gnu-gpl !              */
/****************************************/
/*			www.RedDesert.de			*/
/****************************************/
/*	Project start:			13.09.03	*/
/*	Project last changes:	22.09.03	*/
/****************************************/
/*	ToDo:								*/
/*		-Nothing						*/
/****************************************/


//Includes
#include "RDHighscore.h"

//Functions


RDHighscore::RDHighscore(void){
	this->pEntry	=NULL;		//Pointer resetten
	this->pHead		=NULL;
	return;
}//RDHighscore::RDHighscore(void)



RDHighscore::~RDHighscore(void){
	RDHighscoreEntry	*pPop=NULL;

	if(this->pHead!=NULL) {		//Eventuell belegten speicher freigeben
		delete this->pHead;
		this->pHead	=NULL;
	}

	while(this->pEntry!=NULL) {	//Alle noch gespeicherten Elemente freigeben
		pPop=this->pEntry;
		this->pEntry=this->pEntry->pNext;
		delete pPop;
	}
	return;
}//RDHighscore::~RDHighscore(void)


/*File functions*/



bool	RDHighscore::LoadFile(char chPath[],char chHeaderCode[]){
	RDHighscoreEntry	*pNew	=NULL;
	void				*pData	=NULL;
	FILE				*pFile	=NULL;
	unsigned int		i		=0;


	//Daten checken
	if(chPath==NULL || chHeaderCode==NULL) return false;

	//Alte daten lschen
	this->Reset();

	//File ffnen
	pFile=fopen(chPath,"rb");
	if(pFile==NULL) return false;

	//Speicher fr Kodierten und Decodierten header allokieren
	pData	=new RDHighscoreHeader;	//Speicher fr Kodierten Header allokieren
	if(pData==NULL) {
		fclose(pFile);
		return false;
	}

	this->pHead=new RDHighscoreHeader; //Speicher fr Dekodierten Header allokieren
	if(this->pHead==NULL) {
		fclose(pFile);
		delete (char*)pData;
		return false;
	}

	//Header Laden und Dekodieren
	if(fread(pData,sizeof(RDHighscoreHeader),1,pFile)!=1) {	//Daten lesen
		delete (char*)pData;
		delete this->pHead;
		this->pHead=NULL;
		fclose(pFile);
		return false;
	}

	if(!this->Crypt(this->pHead,pData,sizeof(RDHighscoreHeader),chHeaderCode)) {	//Encrypten
		delete (char*)pData;
		delete this->pHead;
		this->pHead=NULL;
		fclose(pFile);
		return false;
	}	

	delete (char*)pData;	//Unntigen speicher freiegeben
	pData=NULL;

	//Header Checken
	if(this->pHead->uchVersion!=RDHIGHSCORE_VERSION
		|| strcmp(this->pHead->chIDString,RDHIGHSCORE_IDSTRING)!=0) 
	{
		delete this->pHead;
		this->pHead=NULL;
		fclose(pFile);
		return false;
	}
	
	//Highscore Daten laden

	pData	=new RDHighscoreEntry;	//Speicher fr Kodierte daten allokieren
	if(pData==NULL) {
		fclose(pFile);
		this->Reset();
		return false;
	}

	for(i=0;i<this->pHead->uiElements;i++) { //Alle elemente laden
		//Speicher fr Element holen
		pNew	=new RDHighscoreEntry;
		if(pNew==NULL) {
			delete (char*)pData;
			fclose(pFile);
			this->Reset();
			return false;
		}
		//Kodierte daten in Speicher laden
		if(fread(pData,sizeof(RDHighscoreEntry),1,pFile)!=1) {
			fclose(pFile);
			delete pNew;
			delete (char*)pData;
			this->Reset();
			return false;
		}

		//Daten dekodieren
		if(!this->Crypt(pNew,pData,sizeof(RDHighscoreEntry),chHeaderCode)) {
			fclose(pFile);
			delete pNew;
			delete (char*)pData;
			this->Reset();
			return false;
		}
		//Neues Element in Linked List einreihen
		pNew->pNext=this->pEntry;	//Pointer umbiegen
		this->pEntry=pNew;
		pNew		=NULL;			//Pointer zurcksetzten
	}
	//File schlieen und unntig belegten speicher freigeben
	delete (char*)pData;
	fclose(pFile);
	//Liste sortieren
	this->Sort();
	//FERTIG!!!
	return true;
}//bool	RDHighscore::LoadFile(char chPath[],char chHeaderCode[])


unsigned char	RDHighscore::GetCount(void){
	RDHighscoreEntry	*pCnt=NULL;
	unsigned char		uchRet=0;

	//Alle elemente zhlen
	pCnt=this->pEntry;
	while(pCnt!=NULL) {
		pCnt=pCnt->pNext;
		uchRet++;
	}

	return uchRet;
}//unsigned char	RDHighscore::GetCount(void)


bool	RDHighscore::WriteFile(char chPath[],char chHeaderCode[]){
	RDHighscoreEntry		*pEntry	=NULL;
	void					*pData	=NULL;
	FILE					*pFile	=NULL;

	//File ffnen
	pFile=fopen(chPath,"wb");
	if(pFile==NULL) return false;

	//Header aktualisieren
	if(this->pHead==NULL) {	//Fall noch kein header vorhanden
		this->pHead=new RDHighscoreHeader;
		if(pHead==NULL) {
			fclose(pFile);
			return false;
		}
		//Erstellungszeit setzten
		this->pHead->tCreate=time(NULL);
	}

	this->pHead->tChange=time(NULL);	//Vernderungzeit setzten
	sprintf(this->pHead->chIDString,"%s",RDHIGHSCORE_IDSTRING);
	this->pHead->uchVersion	=RDHIGHSCORE_VERSION;
	this->pHead->uiElements	=this->GetCount();

	//Speicher fr Kodierten header allokieren
	pData=new RDHighscoreHeader;
	if(pData==NULL) {
		fclose(pFile);
		return false;
	}

	//Header in neuen speicher kodieren
	if(!this->Crypt(pData,this->pHead,sizeof(RDHighscoreHeader),chHeaderCode)) {
		delete (char*)pData;
		fclose(pFile);
		return false;
	}
	
	//Header in file schreiben
	if(fwrite(pData,sizeof(RDHighscoreHeader),1,pFile)!=1) {
		delete (char*)pData;
		fclose(pFile);
		return false;
	}
	
	//Speicher freigeben
	delete (char*)pData;
	pData=NULL;
	
	//Elemente sortieren
	this->Sort();
	//Speicher fr Kodierte Elemente allokieren
	pData=new RDHighscoreEntry;
	if(pData==NULL) {
		fclose(pFile);
		return false;
	}

	//Alle elemente kodieren und schreiben
	pEntry=this->pEntry;
	while(pEntry!=NULL) {
		//Daten verschlsseln und schreiben
		if(!this->Crypt(pData,pEntry,sizeof(RDHighscoreEntry),chHeaderCode)) {
			delete (char*)pData;
			fclose(pFile);
			return false;
		}
		if(fwrite(pData,sizeof(RDHighscoreEntry),1,pFile)!=1) {
			delete (char*)pData;
			fclose(pFile);
			return false;
		}
		//Nchstes Element anwhlen
		pEntry=pEntry->pNext;
	}
	//Speicher freigeben und file schlieen
	delete (char*)pData;
	fclose(pFile);
	//FERTIG!
	return true;
}//bool	RDHighscore::WriteFile(char chPath[],char chHeaderCode[])



bool	RDHighscore::ResetFile(char chPath[],char chHeaderCode[]){
	RDHighscoreHeader		Head;
	void					*pData	=NULL;
	FILE					*pFile	=NULL;
	

	//File ffnen
	pFile=fopen(chPath,"wb");
	if(pFile==NULL) return false;

	//Header erstellen
	sprintf(Head.chIDString,"%s",RDHIGHSCORE_IDSTRING);
	Head.uchVersion	=RDHIGHSCORE_VERSION;
	Head.uiElements	=0;
	Head.tChange	=time(NULL);
	Head.tCreate	=time(NULL);

	//Speicher fr Kodierten header allokieren
	pData=new RDHighscoreHeader;
	if(pData==NULL) {
		fclose(pFile);
		return false;
	}

	//Header in neuen speicher kodieren
	if(!this->Crypt(pData,this->pHead,sizeof(RDHighscoreHeader),chHeaderCode)) {
		delete (char*)pData;
		fclose(pFile);
		return false;
	}
	
	//Header in file schreiben
	if(fwrite(pData,sizeof(RDHighscoreHeader),1,pFile)!=1) {
		delete (char*)pData;
		fclose(pFile);
		return false;
	}
	
	//Speicher freigeben und file schlieen
	delete (char*)pData;
	fclose(pFile);
	return true;
}//bool	RDHighscore::ResetFile(char chPath[],char chHeaderCode[])



RDHighscoreHeader *RDHighscore::GetHeader(void){
	return this->pHead;
}//RDHighscoreHeader *RDHighscore::GetHeader(void)



/*Highscore functions*/

unsigned char	RDHighscore::Cut(unsigned int uiCut){
	RDHighscoreEntry	*pCut=NULL;
	RDHighscoreEntry	*pBase=NULL;
	
	if(this->pEntry==NULL) return RDHIGHSCORE_ERROR;
	//Liste sortieren
	this->Sort();

	//Zu lschende daten bestimmen
	pBase=this->pEntry;
	while(uiCut>0) {
		pCut=pBase;
		if(pBase==NULL) return RDHIGHSCORE_ERROR;
		pBase=pBase->pNext;
		uiCut--;
	}
	//Element abtrennen und lschen
	if(pCut!=NULL) {	//Pointer umbiegen
		pCut->pNext=NULL;
	}

	while(pBase!=NULL) {
		pCut=pBase;
		pBase=pBase->pNext;
		delete pCut;
		uiCut++;
	}

	return uiCut;
}//unsigned char	RDHighscore::Cut(unsigned int uiCut)


unsigned char	RDHighscore::Add(RDHighscoreEntry HighscoreEntry){


	unsigned char		uchRet=1;
	RDHighscoreEntry	*pNew=NULL;

	//Speicher fr element holen
	pNew=new RDHighscoreEntry;
	if(pNew==NULL) return RDHIGHSCORE_ERROR;


	//Platz des objektes feststellen
	uchRet=this->Check(&HighscoreEntry);

	//Daten kopieren
	memcpy(pNew,&HighscoreEntry,sizeof(RDHighscoreEntry));

	//Element in LL einreihen
	pNew->pNext=this->pEntry;
	this->pEntry=pNew;


	//Liste neu sortieren
	this->Sort();

	
	return uchRet;
}//unsigned char	RDHighscore::Add(RDHighscoreEntry HighscoreEntry)



unsigned char	RDHighscore::Check(RDHighscoreEntry *pHighscoreEntry){
	unsigned char		ret		=0;
	RDHighscoreEntry	*pCmp	=NULL;

	if(pHighscoreEntry==NULL) return RDHIGHSCORE_ERROR;

	//Liste sortieren
	this->Sort();

	//Element zuordnen
	pCmp=this->pEntry;
	ret++;
	while(pCmp!=NULL) {
		if(this->Compare(pHighscoreEntry,pCmp)) break;
		pCmp=pCmp->pNext;
		ret++;
	}

	return ret;
}//unsigned char	RDHighscore::Check(RDHighscoreEntry *pHighscoreEntry)



RDHighscoreEntry *RDHighscore::Get(unsigned char uchPlace){
	RDHighscoreEntry	*pRet=NULL;


	//Liste sortieren
	this->Sort();

	//Liste durchlaufen und Element suchen
	if(uchPlace==0) return this->pEntry;

	pRet=this->pEntry;
	uchPlace--;
	while(uchPlace>0) {
		if(pRet==NULL) return NULL;
		pRet=pRet->pNext;
		uchPlace--;
	}
	//Element zurckgeben
	return pRet;
}


void RDHighscore::Reset(void){
	RDHighscoreEntry	*pPop=NULL;

	if(this->pHead!=NULL) {		//Eventuell belegten speicher freigeben
		delete this->pHead;
		this->pHead	=NULL;
	}

	while(this->pEntry!=NULL) {	//Alle noch gespeicherten Elemente freigeben
		pPop=this->pEntry;
		this->pEntry=this->pEntry->pNext;
		delete pPop;
	}
	return;
}//void RDHighscore::Reset(void)

/*Private Funktionen*/


bool	RDHighscore::Crypt(void *pTarget,void *pSource,unsigned int uiSize,char chCode[]){
	unsigned int				i=0;		//Counter Variablen
	unsigned int				y=0;
	
	if(pTarget==NULL || pSource==NULL || chCode==NULL) return false;	//Bei falschen Parametern return false
	
	//Todo:
	  //!!!!Coding!!!
	for(i=0;i<uiSize;i++) {
		if(y>strlen(chCode)) y=0;
		(*(((char*)pTarget)+i))=(*(((char*)pSource)+i))^chCode[y];
		y++;
	}
	//DEBUG
	//memcpy(pTarget,pSource,uiSize);
	//DEBUG ENDE
	return true;
}//bool	RDHighscore::Crypt(void *pTarget,void *pSource,unsigned int uiSize,char chCode[])


void RDHighscore::Sort(void) {
	RDHighscoreEntry		Tmp;
	RDHighscoreEntry		*pFirst	=NULL;
	RDHighscoreEntry		*pSecond=NULL;

	pSecond=this->pEntry;
	if(pSecond==NULL) return;
	while(pSecond->pNext!=NULL) {
		//Ein element weitergehen
		pFirst=pSecond;
		pSecond=pSecond->pNext;
		
		//Vegleichen
		if(!this->Compare(pFirst,pSecond)) {
			//Diesen ganzen komplizierten mi** braucht man weil ansonsten die pointer berschrieben werden!!!!!
			//Erstes Element puffern
			sprintf(Tmp.chName,"%s",pFirst->chName);
			Tmp.liLevel		=pFirst->liLevel;
			Tmp.liPoints	=pFirst->liPoints;
			Tmp.liTime		=pFirst->liTime;
			Tmp.tCreate		=pFirst->tCreate;
			//2. element auf erstes schreiben
			sprintf(pFirst->chName,"%s",pSecond->chName);
			pFirst->liLevel	=pSecond->liLevel;
			pFirst->liPoints=pSecond->liPoints;
			pFirst->liTime	=pSecond->liTime;
			pFirst->tCreate	=pSecond->tCreate;
			//2. element mit dem puffer berschreiben
			sprintf(pSecond->chName,"%s",Tmp.chName);
			pSecond->liLevel	=Tmp.liLevel;
			pSecond->liPoints	=Tmp.liPoints;
			pSecond->liTime		=Tmp.liTime;
			pSecond->tCreate	=Tmp.tCreate;
			//1. Element wieder als start setzten
			pSecond=this->pEntry;
		}
	}
	return;
}//void RDHighscore::Sort(void)


bool	RDHighscore::Compare(RDHighscoreEntry *pFirst,RDHighscoreEntry *pSecond){
	if(pFirst==NULL) return false;
	if(pSecond==NULL) return true;

	//Muss gefixt werden is im mom ziemlich simpel...
	if(pFirst->liPoints>=pSecond->liPoints) return true;
	
	return false;
}//bool	RDHighscore::Compare(RDHighscoreEntry *pFirst,RDHighscoreEntry *pSecond)


//Hilfe funktionen

RDHighscoreEntry	RDHighscoreCreate(char chName[RDHIGHSCORE_SIZE_NAME],long int liPoints,long int liLevel,long int liTime){
	RDHighscoreEntry	Ret;

	//Alle Werte eintragen
	sprintf(Ret.chName,"%s",chName);
	Ret.liPoints=liPoints;
	Ret.liLevel	=liLevel;
	Ret.liTime	=liTime;
	Ret.tCreate	=time(NULL);

	return Ret;
}//RDHighscoreEntry	RDHighscoreCreate(char chName[RDHIGHSCORE_SIZE_NAME],long int liPoints,long int liLevel,long int liTime){

/****************************************/
/*		By RedDesert (z-man) 2003C		*/
/*		  Please visit our page:		*/
/*			www.RedDesert.de			*/
/****************************************/

//Internal File Version: 0.01

