/****************************************/
/*				RDMen.h					*/
/*	This is a simple menue engine for   */
/*	the RedDesert tetris project (tijot)*/
/*  this source is published under the  */
/*	terms of the GNU GPL! I would be    */
/*	pleased to get some feedback if you */
/*	use this source! Please also visit  */
/*  our hompage.						*/
/****************************************/
/*			www.RedDesert.de			*/
/****************************************/
/*	Project start:			04.01.04	*/
/*	Project last change:	12.01.04	*/
/****************************************/
/*	ToDo:								*/
/*		-Perhaps add other features*g*	*/
/*	DoNe:								*/
/*		-Finish Label class				*/
/*		-Finish Button class			*/
/*		-Finish Keyboard class			*/
/*		-Finish Mouse class				*/
/*		-Finish Input class				*/
/****************************************/
#ifndef _RDMEN_H
#define _RDMEN_H


//includes
#include <stdio.h>
#include "../tfree/tfree.h"


//Defines
#define			RDMEN_NOTINIT			0
#define			RDMEN_STANDART			1
#define			RDMEN_SELECTED			2
#define			RDMEN_PRESSED			3

#define			RDMEN_KLICKNOTHING		0
#define			RDMEN_KLICKRIGHT		0x02
#define			RDMEN_KLICKLEFT			0x04

#define			RDMEN_BLINKDELAY		400

#define			RDMEN_CC				0xff00ff



//Class defines
class			RDMenMouse;
class			RDMenKeyboard;

class			RDMenButton;
class			RDMenInput;
class			RDMenLabel;

class			RDMenResLoader;


//Classes
class			RDMenMouse{
private:
	int				PosX;
	int				PosY;
	int				State;
	int				iOld;

	freeSprite		Grf;
public:
	RDMenMouse(void);
	~RDMenMouse(void);

	bool			Init(char chGrf[]);
	bool			Run(void);
	bool			Draw(void);
	void			Reset(void);
	void			GetState(int *pX,int *pY,int *pS);
};//class			RDMenMouse


class			RDMenKeyboard{
private:
	SDL_Event		event;
	int				PrevState;
	int				iKey;
public:
	RDMenKeyboard(void);
	~RDMenKeyboard(void);

	bool		Init(void);

	bool		Run(void);

	void		GetState(int *iKey);

	void		Reset(void);
};//class			RDMenKeyboard



class			RDMenButton{
private:
	//Mouse pointers
	int				*pPosmX;
	int				*pPosmY;
	int				*pStatem;

	//Graphics
	freeSprite		grfStandart;
	bool			bgrfStandart;
	freeSprite		grfSelected;
	bool			bgrfSelected;
	freeSprite		grfPressed;
	bool			bgrfPressed;

	freeFont		fntStandart;
	bool			bfntStandart;
	freeFont		fntSelected;
	bool			bfntSelected;
	freeFont		fntPressed;
	bool			bfntPressed;
	char			*chText;

	SDL_Rect		BoundRect;

	//Sounds
	CSound			sndSelected;
	bool			bsndSelected;
	CSound			sndPressed;
	bool			bsndPressed;
	//Status variables
	unsigned char	iState;
public:
	RDMenButton(void);
	~RDMenButton(void);

	bool		Init(SDL_Rect BoundRect);
	bool		Run(void);
	bool		Draw(void);
	void		Reset(void);

	int			GetState(void);
	void		SetState(int NewState);

	SDL_Rect	GetRect(void);
	void		SetRect(SDL_Rect BoundRect);

	bool		ActMouse(int *pPosmX,int *pPosmY,int *pStatem);

	bool		ActFntStandart(char chPath[]);
	bool		ActFntSelected(char chPath[]);
	bool		ActFntPressed(char chPath[]);
	bool		ActText(char chText[]);

	bool		ActGrfStandart(char chPath[]);
	bool		ActGrfSelected(char chPath[]);
	bool		ActGrfPressed(char chPath[]);

	bool		ActSndSelected(char chPath[]);
	bool		ActSndPressed(char chPath[]);
};//class			RDMenButton


class			RDMenInput{
private:
	//Mouse pointers
	int				*pPosmX;
	int				*pPosmY;
	int				*pStatem;

	//Keyboard pointer
	int				*iK;

	//Graphics
	freeSprite		grfStandart;
	bool			bgrfStandart;

	freeSprite		grfCursor;

	freeFont		Font;

	SDL_Rect		BoundRect;

	//Timer
	unsigned int	uiBlinkTimer;
	bool			bBlink;
	//Status variables
	unsigned char	iState;

	unsigned int	uiCursor;
	char			*chText;
	unsigned int	uiMaxText;

	RDMenButton		*pAutoPress;
public:
	RDMenInput(void);
	~RDMenInput(void);

	bool		Init(SDL_Rect BoundRect,unsigned int uiMaxText,char chCursor[],char chFont[]);
	bool		Run(void);
	bool		Draw(void);
	void		Reset(void);

	SDL_Rect	GetRect(void);
	void		SetRect(SDL_Rect BoundRect);


	int			GetState(void);
	void		SetState(int NewState);

	bool		ActMouse(int *pPosmX,int *pPosmY,int *pStatem);
	bool		ActKeyboard(int *iK);

	bool		SetAutoPress(RDMenButton *pPress);

	bool		ActGrfStandart(char chPath[]);


	bool		GetText(char pTarget[]);
	bool		SetText(char chText[]);

};//class			RDMenInput


class			RDMenLabel{
private:
	freeSprite		grfStandart;
	bool			bgrfStandart;
	freeFont		fntStandart;
	bool			bfntStandart;
	
	char			*chText;

	SDL_Rect		BoundRect;
public:
	RDMenLabel(void);
	~RDMenLabel(void);

	bool		Init(SDL_Rect BoundRect);
	bool		Draw(void);
	void		Reset(void);

	bool		ActGrfStandart(char chGrf[]);
	bool		ActFntStandart(char chFnt[],char chText[]);
};//class			RDMenLabel


SDL_Rect	RDMenCreateRect(int x,int y,int w,int h);



#endif

/****************************************/
/*		By RedDesert (z-man) 2004C		*/
/*		  Please visit our page:		*/
/*			www.RedDesert.de			*/
/****************************************/
