/****************************************/
/*				RDTetris.cpp			*/
/* A Tetris Engine by RedDesert, you    */
/* may redistribute this source at your */
/* will! But please send us a mail at   */
/* info@reddesert.de or post in our     */
/* board at www.reddesert.de!!			*/
/****************************************/
/*			www.RedDesert.de			*/
/****************************************/
/*	Project start:			12.08.03	*/
/*	Project last change:	20.08.03	*/
/****************************************/
/*	ToDo:								*/
/*		-Nothing						*/
/****************************************/

//Includes
#include "RDTetris.h"		//Tetris header


//////////
//STONES//
//////////

const short RDTStone[RDTS_COUNT][4][7]=
{
	{	//Quadrat
		{
			+0,+0,
			+0,+0,
			+0,+0
		},
		{
			+0,+0,
			+0,+0,
			+0,+0
		},
		{
			+0,+0,
			+0,+0,
			+0,+0
		},
		{
			+0,+0,
			+0,+0,
			+0,+0
		}
	},
	{	//Line
		{
			+1,+1,
			+2,+2,
			-1,-1
		},
		{
			-1,+1,
			-2,+2,
			+1,-1
		},
		{
			-1,-1,
			-2,-2,
			+1,+1
		},
		{
			+1,-1,
			+2,-2,
			-1,+1
		}
	},
	{
		//L-Left
		{
			+1,+1,
			-1,-1,
			-2,0
		},
		{
			-1,+1,
			+1,-1,
			+0,-2
		},
		{
			-1,-1,
			+1,+1,
			+2,0
		},
		{
			+1,-1,
			-1,+1,
			+0,+2
		}
	},
	{
		//L-Right
		{
			+1,+1,
			-1,-1,
			+0,+2
		},
		{
			-1,+1,
			+1,-1,
			-2,+0
		},
		{
			-1,-1,
			+1,+1,
			+0,-2
		},
		{
			+1,-1,
			-1,+1,
			+2,0
		}
	},
	{
		//N-Left
		{
			-1,-1,
			-1,+1,
			+0,+2
		},
		{
			+1,-1,
			-1,-1,
			-2,+0
		},
		{
			+1,+1,
			+1,-1,
			+0,-2
		},
		{
			-1,+1,
			+1,+1,
			+2,0
		}
	},
	{
		//N-Right
		{
			+1,+1,
			-2,+0,
			-1,+1
		},
		{
			-1,+1,
			+0,-2,
			-1,-1
		},
		{
			-1,-1,
			+2,+0,
			+1,-1
		},
		{
			+1,-1,
			+0,+2,
			+1,+1
		}
	},
	{
		//T
		{
			+1,+1,
			-1,-1,
			-1,+1
		},
		{
			-1,+1,
			+1,-1,
			-1,-1
		},
		{
			-1,-1,
			+1,+1,
			+1,-1
		},
		{
			+1,-1,
			-1,+1,
			+1,+1
		}
	}		
};

//////////////////
//class RDTetris//
//////////////////			
		

//////////////////////////
//Constructor/Destructor//
//////////////////////////


RDTetris::RDTetris(void) {	//Constructor
	int x;
	int y;
	//Reset Counter and Variables
	this->uiDelay	=0;		//Reset Movement Delay
	this->iLines	=0;		//Reset Line Count
	this->iNextType	=0;		//Reset Next Type
	this->iPoints	=0;		//Reset Points
	this->iType		=0;		//Reset actual Points
	this->uiTimer	=0;		//Reset Timer
	this->uchFlags	=RDTF_STANDART;	//Reset Flags
	this->iLines	=0;		//Reset Last lines
	this->iLPoints	=0;		//Reset Last points

	//Reset Arrays
	for(x=0;x<=RDTF_WIDTH;x++) {	//Walk through all x values
		for(y=0;y<=RDTF_HEIGHT;y++) { //Walk through all y values
			this->pField[x][y]=NULL; //Reset x,y value
		}
	}
	for(x=0;x<=RDTS_QUADSTONE;x++) {	//Walk through all Quads in the stone
		this->pStone[x]=NULL;	//Reset them
	}
	for(x=0;x<=RDTS_STONEC;x++) { //Walk through all Coordinates
		this->iaStoneCoord[x]=0; //Reset them
	} 
	this->pFieldGrafic=NULL;
	for(x=0;x<=RDTG_SPRITES;x++) {
		this->pSprites[x]=NULL;
	}
	for(x=0;x<=RDTM_COUNT;x++) {
		this->pSounds[x]=NULL;
	}
	return;  //Return
}//RDTetris::RDTetris(void)


RDTetris::~RDTetris(void) { //Destructor
	this->Reset();	//Use internal Reset function
	return; //Return
}//RDTetris::~RDTetris(void)


//////////////////////
//Internal functions//
//////////////////////


bool	RDTetris::IsBound(int x,int y) {		//Is already a quad at pos x,y
	if(x<0 || y<0 || x>RDTF_WIDTH || y>RDTF_HEIGHT) return true; //If the stuff is out of the array bounds
	if(this->pField[x][y]==NULL) return false;		//If there's no quad return false
	return true;	//Else return true
}//bool	RDTetris::IsBound(int x,int y)


bool	RDTetris::IsFilled(int y) {				//Is line y completely filled with quads?
	if(y<0 || y>RDTF_HEIGHT) return false;	//If it isn't in the array beware of accesing it...
	for(int x=0;x<=RDTF_WIDTH;x++) {	//Walk through the whole line
		if(this->pField[x][y]==NULL) return false; //If any Field is NULL the line can't be filled completely -->return false
	}
	return true; //If the programm reaches this the whole line is filled return true
}//bool	RDTetris::IsFilled(int y)


bool	RDTetris::IsTouched(int y) {		//Is line y touched by a quad?
	if(y<0 || y>RDTF_HEIGHT) return false;	//If it isn't in the array beware of accesing it...
	for(int x=0;x<=RDTF_WIDTH;x++) {	//Walk through the whole line
		if(this->pField[x][y]!=NULL) return true; //If any Field is not NULL the line can't be clear at all -->return true
	}
	return false; //If the programm reaches this, there's no Field touched at all-->return true
}//bool	RDTetris::IsTouched(int y)


bool	RDTetris::SetStone(int Colour) {		//Sets stone by using the iaStone Coords
	for(int i=0;i<=RDTS_QUADSTONE;i++) {		//Walk through all Stone quads
		this->pStone[i]=new RDQUAD;		//Allocate Memory
		if(this->pStone[i]==NULL) return false;		//Is memory really allocated?If not return false
		if(this->pField[this->iaStoneCoord[i*2]][this->iaStoneCoord[i*2+1]]!=NULL) {	//This very stupid code checks if theres already a stone on the desired position
			delete this->pStone[i];		//Free allocated memory
			this->pStone[i]=NULL;		//Reset pStone[i] pointer
			return false;		//Return false
		}
		this->pStone[i]->uchSprite	=Colour;
		this->pStone[i]->uchFlags	=RDTF_STANDART;
		this->pField[this->iaStoneCoord[i*2]][this->iaStoneCoord[i*2+1]]=this->pStone[i]; //This one sets the "global" field pointer to the allocated memory of the pStone[i] pointer
	}
	return true; //Return true
}//bool	RDTetris::SetStone(int Colour)


void	RDTetris::LockStone(void) {		//Lock stone so the engine doesn't move it
	for(int i=0;i<=RDTS_QUADSTONE;i++)  {	//Walk through all quads in the stone
		this->pStone[i]->uchFlags|=RDTF_LOCKED; //Apply the RDTF_LOCKED flag
	}
	return;	//Return
}//void	RDTetris::LockStone(void)


void	RDTetris::CrackLine(int y) {	//Kill all quads in line y
	if(y<0 || y>RDTF_HEIGHT) return;	//If it isn't in the array beware of accesing it...
	for(int x=0;x<=RDTF_WIDTH;x++) { //Walk through all X Values
		if(this->pField[x][y]!=NULL) {	//Is there anything on the field?
			delete this->pField[x][y];	//Delete it
			this->pField[x][y]=NULL;	//Reset the pointer
		}
	}
	return;	//Return;
}//void	RDTetris::CrackLine(int y)


void	RDTetris::Move(void) {		//Moves all unlocked stones down and locks them
	for(int x=0;x<=RDTF_WIDTH;x++) { //Walk through all X Values
		for(int y=RDTF_HEIGHT-1;y>=0;y--) { //Walk backwards through all Y Values but the first one
			if(this->pField[x][y]!=NULL && !(this->pField[x][y]->uchFlags & RDTF_LOCKED)) { //If field element is filled and unlocked
				if(this->pField[x][y+1]!=NULL) {	//If the desired target is filled (that shouldn't happen...but it woudl cause memory lags not to check it....)
					this->pField[x][y]->uchFlags |=RDTF_LOCKED;
					continue;
				}
				this->pField[x][y+1]=this->pField[x][y];	//Set Target pointer to real pointer
				this->pField[x][y]=NULL;					//Set real pointer NULL
				this->pField[x][y+1]->uchFlags |=RDTF_LOCKED;//Lock the new position
			}
		}
	}
	return;
}//void	RDTetris::Move(void)


void	RDTetris::UnlockField(int y){		//Unlocks all Quads over the y coord
	if(y<0 || y>RDTF_HEIGHT) return;	//If it isn't in the array beware of accesing it...
	for(int x=0;x<=RDTF_WIDTH;x++) { //Walk through all X Values
		for(int ay=0;ay<=y;ay++) {		//Walk through all Selected Y Values
			if(this->pField[x][ay]!=NULL) { //If theres a quad on the position and if the quad's locked...
				this->pField[x][ay]->uchFlags	=RDTF_STANDART;	//Set Standart flag -->ToDo: remove Flag
			}
		}
	}
	return;		//Return
}//void	RDTetris::UnlockField(int y)


bool	RDTetris::TransformXY(int difx,int dify){	//Transforms actual stone on x and y axes
	int i;
	for(i=0;i<=RDTS_QUADSTONE;i++) {	//Walk through all Stone quads
		this->pField[this->iaStoneCoord[i*2]][this->iaStoneCoord[i*2+1]]=NULL; //Remove Stone from field pointer list
	}
	for(i=0;i<=RDTS_QUADSTONE;i++) {	//Walk through all Stone quads
		if(this->IsBound(this->iaStoneCoord[i*2]+difx,this->iaStoneCoord[i*2+1]+dify)){
			for(i=0;i<=RDTS_QUADSTONE;i++) {
				this->pField[this->iaStoneCoord[i*2]][this->iaStoneCoord[i*2+1]]=this->pStone[i];	//Reset field pointer list
			}
			return false; //to check if there would be quads on the transformed coordinates ,if yes return false
		}
	}
	for(i=0;i<=RDTS_QUADSTONE;i++) {	//Walk through all Stone quads
		this->pField[this->iaStoneCoord[i*2]+difx][this->iaStoneCoord[i*2+1]+dify]=this->pStone[i]; //Set the Target Pointer to the value of the selected stone pointer
		this->iaStoneCoord[i*2]		+=difx;
		this->iaStoneCoord[i*2+1]	+=dify;
	}
	return true;	//Return true
}//bool	RDTetris::TransformXY(int difx,int dify)


bool	RDTetris::TransformRot(void) { //Transforms actual stone with rotate	
	int i;
	for(i=0;i<=RDTS_QUADSTONE;i++) {	//Walk through all Stone quads
		this->pField[this->iaStoneCoord[i*2]][this->iaStoneCoord[i*2+1]]=NULL; //Remove Stone from field pointer list
	}
	for(i=1;i<=RDTS_QUADSTONE;i++) {
		if(this->IsBound(this->iaStoneCoord[i*2]+RDTStone[this->iType][this->iaStoneCoord[8]][(i-1)*2],this->iaStoneCoord[i*2+1]+RDTStone[this->iType][this->iaStoneCoord[8]][(i-1)*2+1])) {
			for(i=0;i<=RDTS_QUADSTONE;i++) {
				this->pField[this->iaStoneCoord[i*2]][this->iaStoneCoord[i*2+1]]=this->pStone[i];	//Reset field pointer list
			}
			return false; //to check if there would be quads on the transformed coordinates ,if yes return false
		}
	}
	this->pField[this->iaStoneCoord[0]][this->iaStoneCoord[1]]=this->pStone[0];
	for(i=1;i<=RDTS_QUADSTONE;i++) {
		this->pField[this->iaStoneCoord[i*2]+RDTStone[this->iType][this->iaStoneCoord[8]][(i-1)*2]][this->iaStoneCoord[i*2+1]+RDTStone[this->iType][this->iaStoneCoord[8]][(i-1)*2+1]]=this->pStone[i];
		this->iaStoneCoord[i*2]		=this->iaStoneCoord[i*2]+RDTStone[this->iType][this->iaStoneCoord[8]][(i-1)*2];
		this->iaStoneCoord[i*2+1]	=this->iaStoneCoord[i*2+1]+RDTStone[this->iType][this->iaStoneCoord[8]][(i-1)*2+1];
	}
	this->iaStoneCoord[8]++;
	if(this->iaStoneCoord[8]>3) {
		this->iaStoneCoord[8]=0;
	}
	return true;
}//bool	RDTetris::TransformRot(int rot)
		


////////////////////
//Public functions//
////////////////////


bool	RDTetris::Init(int iSpeed) {	//Init Game
	if(this->uchFlags & RDTF_RUNNING) return false; //If a game is running abort
	this->Reset();		//Reset everything

	if(iSpeed>RDTG_DELAY) return false;	//Accept no negativ values
	this->uiDelay	=RDTG_DELAY-iSpeed;	//Work out real delay


	srand(SDL_GetTicks());				//Set randoom seed

	this->iNextType	=rand()%RDTS_COUNT;	//Work out the first stone type by rand()
	this->uchFlags	=RDTF_RUNNING;		//Set class status to running

	return true;	//Return true
}//bool	RDTetris::Init(int iSpeed)


//Sound


bool	RDTetris::SetSound(int ArrayIndex,CSound *pPointer) {
	if(ArrayIndex<0 || ArrayIndex>RDTM_COUNT) return false;

	this->pSounds[ArrayIndex]=pPointer;
	return true;
}//bool	RDTetris::SetSound(int ArrayIndex,CSound *pPointer)


CSound	*RDTetris::GetSound(int ArrayIndex) {
	if(ArrayIndex<0 || ArrayIndex>RDTM_COUNT) return NULL;

	return this->pSounds[ArrayIndex];
}//CSound	*RDTetris::GetSound(int ArrayIndex)


//Graphic

bool	RDTetris::SetGraphic(int ArrayIndex,freeSprite *pPointer) { //Sets a Graphic into the class array
	if(ArrayIndex<(-1) || ArrayIndex>RDTG_SPRITES) return false;
	if(ArrayIndex==(-1)) {
		this->pFieldGrafic=pPointer;
	}
	this->pSprites[ArrayIndex]=pPointer;
	return true;
}//bool	RDTetris::SetGraphic(int ArrayIndex,freeSprite *pPointer)


freeSprite *RDTetris::GetGraphic(int ArrayIndex) {	//Returns the graphic value set by the user
	if(ArrayIndex<(-1) || ArrayIndex>RDTG_SPRITES) return NULL;
	if(ArrayIndex==(-1)) {
		return this->pFieldGrafic;
	}
	return this->pSprites[ArrayIndex];
}//freeSprite *RDTetris::GetGraphic(int ArrayIndex)


bool	RDTetris::Run(int Action) { //Run game and perform Action
	if(!(this->uchFlags & RDTF_RUNNING)) return false; //If there isn't a running game return false
	//Reset lifetime counters
	this->iLPoints	=0;
	this->iLLines	=0;
	if(!(this->uchFlags & RDTF_STONE)) {	//If theres no falling stone...
		//Check field for full lines
		for(int i=0;i<=RDTF_HEIGHT;i++) {	//Walk through all lines
			if(this->IsFilled(i)) {	//If line's full
				this->CrackLine(i); //Destroy the line
				this->UnlockField(i); //Unlock the field above
				this->Move();			//Move unlocked stones
				this->iLLines++;
			}
		}
		//Point handling
		if(this->iLLines!=0) {
			this->iLines+=this->iLLines;			//Refresh Lines Counter
			this->iLPoints=this->iLLines*this->iLLines*RDTG_PPL;
			this->iPoints+=this->iLPoints; //Refresh points
			//If sounds aviable play it
			if(this->pSounds[RDTM_CRACKLINE]!=NULL) {
				this->pSounds[RDTM_CRACKLINE]->Play();
			}
		}

		//Move lines	
		//Check if player lost
		if(this->IsTouched(0)) {	//If the highest line is touched
			this->uchFlags	=RDTF_LOST;
			return false;
		}
		//Check delay
		if(SDL_GetTicks()-this->uiTimer>this->uiDelay*2) {		//Wait 2* standart delay before setup a new stone
			this->uiTimer	=SDL_GetTicks(); //Reset Timer
			//Create new Stone
			switch(this->iNextType) {	//Switch between the different types
			case RDTS_QUAD:		//A Quad
				this->iaStoneCoord[0]	=RDTF_STARTX;
				this->iaStoneCoord[1]	=RDTF_STARTY;			//		@@
																//		@@
				this->iaStoneCoord[2]	=RDTF_STARTX+1;			
				this->iaStoneCoord[3]	=RDTF_STARTY;			

				this->iaStoneCoord[4]	=RDTF_STARTX;
				this->iaStoneCoord[5]	=RDTF_STARTY+1;

				this->iaStoneCoord[6]	=RDTF_STARTX+1;
				this->iaStoneCoord[7]	=RDTF_STARTY+1;

				//Reset Animation counter
				this->iaStoneCoord[8]	=1;

				break;
			case RDTS_LINE:		//A Line
				this->iaStoneCoord[0]	=RDTF_STARTX;
				this->iaStoneCoord[1]	=RDTF_STARTY;			//	   @@@@
																//	
				this->iaStoneCoord[2]	=RDTF_STARTX+1;			
				this->iaStoneCoord[3]	=RDTF_STARTY;			

				this->iaStoneCoord[4]	=RDTF_STARTX+2;
				this->iaStoneCoord[5]	=RDTF_STARTY;

				this->iaStoneCoord[6]	=RDTF_STARTX-1;
				this->iaStoneCoord[7]	=RDTF_STARTY;	
				
				//Reset Animation counter
				this->iaStoneCoord[8]	=1;

				break;
			case RDTS_L_LEFT:	//A left orientated L shaped stone
				this->iaStoneCoord[0]	=RDTF_STARTX;
				this->iaStoneCoord[1]	=RDTF_STARTY;			//	   @@@
																//	   @ 
				this->iaStoneCoord[2]	=RDTF_STARTX+1;			
				this->iaStoneCoord[3]	=RDTF_STARTY;			

				this->iaStoneCoord[4]	=RDTF_STARTX-1;
				this->iaStoneCoord[5]	=RDTF_STARTY;

				this->iaStoneCoord[6]	=RDTF_STARTX-1;
				this->iaStoneCoord[7]	=RDTF_STARTY+1;	
				
				//Reset Animation counter
				this->iaStoneCoord[8]	=1;

				break;
			case RDTS_L_RIGHT:	//A right orientated L shaped stone
				this->iaStoneCoord[0]	=RDTF_STARTX;
				this->iaStoneCoord[1]	=RDTF_STARTY;			//	   @@@
																//		 @ 
				this->iaStoneCoord[2]	=RDTF_STARTX+1;			
				this->iaStoneCoord[3]	=RDTF_STARTY;			

				this->iaStoneCoord[4]	=RDTF_STARTX-1;
				this->iaStoneCoord[5]	=RDTF_STARTY;

				this->iaStoneCoord[6]	=RDTF_STARTX+1;
				this->iaStoneCoord[7]	=RDTF_STARTY+1;	
				
				//Reset Animation counter
				this->iaStoneCoord[8]	=1;

				break;
			case RDTS_N_LEFT: //A left orientated N Shaped stone
				this->iaStoneCoord[0]	=RDTF_STARTX;
				this->iaStoneCoord[1]	=RDTF_STARTY;			//   @@
																//	@@ 
				this->iaStoneCoord[2]	=RDTF_STARTX-1;			
				this->iaStoneCoord[3]	=RDTF_STARTY;			

				this->iaStoneCoord[4]	=RDTF_STARTX;
				this->iaStoneCoord[5]	=RDTF_STARTY+1;

				this->iaStoneCoord[6]	=RDTF_STARTX+1;
				this->iaStoneCoord[7]	=RDTF_STARTY+1;	
				
				//Reset Animation counter
				this->iaStoneCoord[8]	=1;
				break;
			case RDTS_N_RIGHT: //A right orientated N Shaped stone
				this->iaStoneCoord[0]	=RDTF_STARTX;
				this->iaStoneCoord[1]	=RDTF_STARTY;			//   @@
																//	  @@ 
				this->iaStoneCoord[2]	=RDTF_STARTX+1;			
				this->iaStoneCoord[3]	=RDTF_STARTY;			

				this->iaStoneCoord[4]	=RDTF_STARTX-1;
				this->iaStoneCoord[5]	=RDTF_STARTY+1;

				this->iaStoneCoord[6]	=RDTF_STARTX;
				this->iaStoneCoord[7]	=RDTF_STARTY+1;	
				
				//Reset Animation counter
				this->iaStoneCoord[8]	=1;
				break;
			case RDTS_T:		//A T-Shaped stone
				this->iaStoneCoord[0]	=RDTF_STARTX;
				this->iaStoneCoord[1]	=RDTF_STARTY;			//	   @@@
																//		@
				this->iaStoneCoord[2]	=RDTF_STARTX+1;			
				this->iaStoneCoord[3]	=RDTF_STARTY;			

				this->iaStoneCoord[4]	=RDTF_STARTX-1;
				this->iaStoneCoord[5]	=RDTF_STARTY;

				this->iaStoneCoord[6]	=RDTF_STARTX;
				this->iaStoneCoord[7]	=RDTF_STARTY+1;

				//Reset Animation counter
				this->iaStoneCoord[8]	=1;

				break;
			default:break;
			}
			this->uiSelSprite++;
			if(this->uiSelSprite>RDTG_SPRITES) this->uiSelSprite=0;

			if(!this->SetStone(this->uiSelSprite)) {		//Set stone if failed
				this->uchFlags	=RDTF_LOST;	//Player has lost
				return false;	//Return false
			}

			this->uchFlags |=RDTF_STONE; //Set Stone flag
			this->LockStone();		//Lock the stone so only the run function moves it

			this->iPoints+=RDTG_PPS;	//Add points for a stone

			this->iType	=this->iNextType;	//Set Actual Type

			this->iNextType=rand()%RDTS_COUNT;//Create new randoom Type

		}
	}
	else {		//If there's a falling stone
		//Check delay
		if(SDL_GetTicks()-this->uiITimer>RDTG_CONTROLLERDELAY) {	//If delay has passed...
			this->uiITimer	=SDL_GetTicks(); //Reset Timer
			//Check Input

			if(Action & RDTA_LEFT) {		//If Action left is set
				this->TransformXY(-1,0);	//Move stone left
			}

			if(Action & RDTA_RIGHT) {		//If Action right is set
				this->TransformXY(+1,0);	//Move stone right
			}

			if(Action & RDTA_ROTATE) {		//If Action rotate is set
				this->TransformRot();		//Rotate stone 90 degrees right
			}

			if(Action & RDTA_SPEEDUP) {		//If Speedup is set
				if(!(this->uchFlags & RDTF_SPEEDUP)) { //Check if already set
					this->uchFlags|=RDTF_SPEEDUP;//Set Speedup flag
				}
			}
	
		}
		else {
			if(!(Action & RDTA_SPEEDUP) && this->uchFlags & RDTF_SPEEDUP) {	//Check if Speedup is pressed and activated
				this->uchFlags	^=RDTF_SPEEDUP;
				
			}
		}
		//Move
		if(this->uchFlags	& RDTF_SPEEDUP) {
			if(SDL_GetTicks()-this->uiTimer>(this->uiDelay/RDTG_SUBMULTI)) {	//If delay has passed...
				this->uiTimer	=SDL_GetTicks();
				this->uchFlags^=RDTF_SPEEDUP;
				if(!this->TransformXY(0,1)){	//Move the Stone down
					this->uchFlags=RDTF_RUNNING; //Remove stone flag
					for(int i=0;i<=RDTS_QUADSTONE;i++) { //Walk through all stone quads
						this->pStone[i]=NULL;	//Release them
					}
					//If sounds aviable play it!
					if(this->pSounds[RDTM_STONEDOWN]!=NULL) {
						this->pSounds[RDTM_STONEDOWN]->Play();
					}
					
					return true;	//Return true
				}
			}

		}
		else {
			if(SDL_GetTicks()-this->uiTimer>this->uiDelay) {	//If delay has passed...
				this->uiTimer	=SDL_GetTicks();
				if(!this->TransformXY(0,1)){	//Move the Stone down
					this->uchFlags=RDTF_RUNNING; //Remove stone flag
					for(int i=0;i<=RDTS_QUADSTONE;i++) { //Walk through all stone quads
						this->pStone[i]=NULL;	//Release them
					}
					//If sounds aviable play it!
					if(this->pSounds[RDTM_STONEDOWN]!=NULL) {
						this->pSounds[RDTM_STONEDOWN]->Play();
					}
					
					return true;	//Return true
				}
			}
		}



	}
	return true; //Return true
}//bool	RDTetris::Run(int Action)


int		RDTetris::GetLastLines(void) {		//Returns amount of lines cracked in the last run call
	return this->iLLines;
}//int		RDTetris::GetLastLines(void)


int		RDTetris::GetLastPoints(void) {		//Returns amount of points earned in the last run call
	return this->iLPoints;
}//int		RDTetris::GetLastPoints(void)


bool	RDTetris::Lost() {		//Checks if player has lost or not
	if(this->uchFlags & RDTF_LOST) return true;	//If Lost flag is set return true
	return false;	//else return false
}//bool	RDTetris::Lost()


void	RDTetris::Reset(void) {		//Reset Game and Release all used memory
	//Free Allocated memory & Pointer
	int x;
	for(x=0;x<=RDTF_WIDTH;x++) {	//Walk through all X Coords
		for(int y=0;y<=RDTF_HEIGHT;y++) { //Walk through all Y Coords
			if(this->pField[x][y]!=NULL) { //If the field is filled
				delete this->pField[x][y];	//Free memory
				this->pField[x][y]=NULL;	//Reset pointer
			}
		}
	}
	for(x=0;x<=RDTS_QUADSTONE;x++)	{	//Walk through all Stone pointer
		this->pStone[x]=NULL;	//Pointer can't be right so reset them
	}
	for(x=0;x<=RDTG_SPRITES;x++) {	//Walk through all Sprite pointers
		this->pSprites[x]=NULL;	//Reset them
	}
	this->pFieldGrafic=NULL;

	for(x=0;x<=RDTM_COUNT;x++) {
		this->pSounds[x]=NULL;
	}
	//Reset Variables
	this->uiDelay	=0;				//Reset Movement Delay
	this->iLines	=0;				//Reset Line Count
	this->iNextType	=0;				//Reset Next Type
	this->iPoints	=0;				//Reset Points
	this->iLines	=0;				//Reset Last lines
	this->iLPoints	=0;				//Reset Last points
	this->iType		=0;				//Reset actual Points
	this->uiTimer	=0;				//Reset Timer
	this->uiITimer	=0;				//Reset Action timer
	this->uchFlags	=RDTF_STANDART;	//Reset Flags


	return ; //Return
}//void	RDTetris::Reset(void)



void RDTetris::Restart(int iSpeed) {
	//Free Allocated memory & Pointer
	int x;
	for(x=0;x<=RDTF_WIDTH;x++) {	//Walk through all X Coords
		for(int y=0;y<=RDTF_HEIGHT;y++) { //Walk through all Y Coords
			if(this->pField[x][y]!=NULL) { //If the field is filled
				delete this->pField[x][y];	//Free memory
				this->pField[x][y]=NULL;	//Reset pointer
			}
		}
	}
	for(x=0;x<=RDTS_QUADSTONE;x++)	{	//Walk through all Stone pointer
		this->pStone[x]=NULL;	//Pointer can't be right so reset them
	}
	//Reset Variables
	if(iSpeed>RDTG_DELAY) return;	//Accept no negativ values
	this->uiDelay	=RDTG_DELAY-iSpeed;	//Work out real delay


	srand(SDL_GetTicks());				//Set randoom seed

	this->iNextType	=rand()%RDTS_COUNT;	//Work out the first stone type by rand()
	this->uchFlags	=RDTF_RUNNING;		//Set class status to running

	this->uiDelay	=RDTG_DELAY-iSpeed;//Refresh Movement Delay
	this->iLines	=0;				//Reset Line Count
	this->iPoints	=0;				//Reset Points
	this->iType		=0;				//Reset actual Points
	this->uiTimer	=0;				//Reset Timer
	this->uiITimer	=0;				//Reset Action timer

	return ; //Return
}//void	RDTetris::Restart(iSpeed)


int	RDTetris::GetLines(void) {		//Returns number of cracked lines
	return this->iLines;	//Return lines
}//int	RDTetris::GetLines(void)


void RDTetris::SetLines(int iLines) { //Sets Lines
	this->iLines=iLines;
	return;
}//void RDTetris::SetLines(int iLines)

int RDTetris::GetPoints(void) {		//Returns number of won points
	return this->iPoints;	//Return points
}//int RDTetris::GetPoints(void)


void RDTetris::SetPoints(int iPoints) { //Sets Points
	this->iPoints=iPoints;
	return;
}//void RDTetris::SetPoints(int iPoints)


int RDTetris::GetSpeed(void) {		//Returns the given speed value
	return RDTG_DELAY-this->uiDelay;	//Return speed
}//int RDTetris::GetSpeed(void)


int RDTetris::GetStoneType(void) {	//Returns the ID of the actual falling stone
	return this->iType;
}//int RDTetris::GetStoneType(void)


int RDTetris::GetNextStoneType(void) { //Returns the ID if the next falling stone
	return this->iNextType;
}//int RDTetris::GetNextStoneType(void)


void RDTetris::SetSpeed(int iSpeed) {	//Sets a new speed value
	if(iSpeed>RDTG_DELAY || !(this->uchFlags & RDTF_RUNNING)) return;	//Accept no negativ values

	this->uiDelay	=RDTG_DELAY-iSpeed;	//Work out real delay
	return;
}//void RDTetris::SetSpeed(int iSpeed)



/////////////////
//Draw function//
/////////////////

bool	RDTetris::Draw(int relX,int relY) {	//Draws the whole field
	for(int x=0;x<=RDTF_WIDTH;x++) {
		for(int y=0;y<=RDTF_HEIGHT;y++) {
			if(this->pField[x][y]!=NULL) {
				if(this->pSprites[this->pField[x][y]->uchSprite]!=NULL) {
					if(!this->pSprites[this->pField[x][y]->uchSprite]->draw(relX+(x*RDTG_XOFF),relY+(y*RDTG_YOFF)))return false;
				}
			}
			else {
				if(this->pFieldGrafic!=NULL) {
					if(!this->pFieldGrafic->draw(relX+(x*RDTG_XOFF),relY+(y*RDTG_YOFF))) return false;
				}
			}

		}
	}
	return true;
}//bool	RDTetris::Draw(int relX,int relY)



bool	RDTetris::DrawActual(int relX,int relY) {	//Draws the actual Stone
	switch(this->iType) {
	case RDTS_QUAD:
		if(!this->pSprites[this->uiSelSprite]->draw(relX,relY) ||
		!this->pSprites[this->uiSelSprite]->draw(relX+RDTG_XOFF,relY) ||
		!this->pSprites[this->uiSelSprite]->draw(relX,relY+RDTG_YOFF) ||
		!this->pSprites[this->uiSelSprite]->draw(relX+RDTG_XOFF,relY+RDTG_YOFF)) return false;
		break;
	case RDTS_LINE:
		if(!this->pSprites[this->uiSelSprite]->draw(relX,relY) ||
		!this->pSprites[this->uiSelSprite]->draw(relX+RDTG_XOFF,relY) ||
		!this->pSprites[this->uiSelSprite]->draw(relX+RDTG_XOFF*2,relY) ||
		!this->pSprites[this->uiSelSprite]->draw(relX-RDTG_XOFF,relY)) return false;
		break;
	case RDTS_L_LEFT:
		if(!this->pSprites[this->uiSelSprite]->draw(relX,relY) ||
		!this->pSprites[this->uiSelSprite]->draw(relX+RDTG_XOFF,relY) ||
		!this->pSprites[this->uiSelSprite]->draw(relX+RDTG_XOFF*2,relY) ||
		!this->pSprites[this->uiSelSprite]->draw(relX,relY+RDTG_YOFF)) return false;
		break;
	case RDTS_L_RIGHT:
		if(!this->pSprites[this->uiSelSprite]->draw(relX,relY) ||
		!this->pSprites[this->uiSelSprite]->draw(relX+RDTG_XOFF,relY) ||
		!this->pSprites[this->uiSelSprite]->draw(relX+RDTG_XOFF*2,relY) ||
		!this->pSprites[this->uiSelSprite]->draw(relX+RDTG_XOFF*2,relY+RDTG_YOFF)) return false;
		break;
	case RDTS_T:
		if(!this->pSprites[this->uiSelSprite]->draw(relX,relY) ||
		!this->pSprites[this->uiSelSprite]->draw(relX+RDTG_XOFF,relY) ||
		!this->pSprites[this->uiSelSprite]->draw(relX-RDTG_XOFF,relY) ||
		!this->pSprites[this->uiSelSprite]->draw(relX,relY+RDTG_YOFF)) return false;
		break;
	case RDTS_N_LEFT:
		if(!this->pSprites[this->uiSelSprite]->draw(relX,relY) ||
		!this->pSprites[this->uiSelSprite]->draw(relX-RDTG_XOFF,relY) ||
		!this->pSprites[this->uiSelSprite]->draw(relX,relY+RDTG_YOFF) ||
		!this->pSprites[this->uiSelSprite]->draw(relX+RDTG_XOFF,relY+RDTG_YOFF)) return false;
		break;
	case RDTS_N_RIGHT:
		if(!this->pSprites[this->uiSelSprite]->draw(relX,relY) ||
		!this->pSprites[this->uiSelSprite]->draw(relX+RDTG_XOFF,relY) ||
		!this->pSprites[this->uiSelSprite]->draw(relX,relY+RDTG_YOFF) ||
		!this->pSprites[this->uiSelSprite]->draw(relX-RDTG_XOFF,relY+RDTG_YOFF)) return false;
		break;
	default:
		return false;
		break;
	}
	return true;
}//bool	RDTetris::DrawActual(int relX,int relY)


bool	RDTetris::DrawNext(int relX,int relY) {	//Draws the next Stone
	unsigned int uiSelSpriteVirt;
	
	uiSelSpriteVirt=this->uiSelSprite;
	uiSelSpriteVirt++;
	if(uiSelSpriteVirt>RDTG_SPRITES) uiSelSpriteVirt=0;

	switch(this->iNextType) {
	case RDTS_QUAD:
		if(!this->pSprites[uiSelSpriteVirt]->draw(relX,relY) ||
		!this->pSprites[uiSelSpriteVirt]->draw(relX+RDTG_XOFF,relY) ||
		!this->pSprites[uiSelSpriteVirt]->draw(relX,relY+RDTG_YOFF) ||
		!this->pSprites[uiSelSpriteVirt]->draw(relX+RDTG_XOFF,relY+RDTG_YOFF)) return false;
		break;
	case RDTS_LINE:
		if(!this->pSprites[uiSelSpriteVirt]->draw(relX,relY) ||
		!this->pSprites[uiSelSpriteVirt]->draw(relX+RDTG_XOFF,relY) ||
		!this->pSprites[uiSelSpriteVirt]->draw(relX+RDTG_XOFF*2,relY) ||
		!this->pSprites[uiSelSpriteVirt]->draw(relX-RDTG_XOFF,relY)) return false;
		break;
	case RDTS_L_LEFT:
		if(!this->pSprites[uiSelSpriteVirt]->draw(relX,relY) ||
		!this->pSprites[uiSelSpriteVirt]->draw(relX+RDTG_XOFF,relY) ||
		!this->pSprites[uiSelSpriteVirt]->draw(relX+RDTG_XOFF*2,relY) ||
		!this->pSprites[uiSelSpriteVirt]->draw(relX,relY+RDTG_YOFF)) return false;
		break;
	case RDTS_L_RIGHT:
		if(!this->pSprites[uiSelSpriteVirt]->draw(relX,relY) ||
		!this->pSprites[uiSelSpriteVirt]->draw(relX+RDTG_XOFF,relY) ||
		!this->pSprites[uiSelSpriteVirt]->draw(relX+RDTG_XOFF*2,relY) ||
		!this->pSprites[uiSelSpriteVirt]->draw(relX+RDTG_XOFF*2,relY+RDTG_YOFF)) return false;
		break;
	case RDTS_T:
		if(!this->pSprites[uiSelSpriteVirt]->draw(relX,relY) ||
		!this->pSprites[uiSelSpriteVirt]->draw(relX+RDTG_XOFF,relY) ||
		!this->pSprites[uiSelSpriteVirt]->draw(relX-RDTG_XOFF,relY) ||
		!this->pSprites[uiSelSpriteVirt]->draw(relX,relY+RDTG_YOFF)) return false;
		break;
	case RDTS_N_LEFT:
		if(!this->pSprites[uiSelSpriteVirt]->draw(relX,relY) ||
		!this->pSprites[uiSelSpriteVirt]->draw(relX-RDTG_XOFF,relY) ||
		!this->pSprites[uiSelSpriteVirt]->draw(relX,relY+RDTG_YOFF) ||
		!this->pSprites[uiSelSpriteVirt]->draw(relX+RDTG_XOFF,relY+RDTG_YOFF)) return false;
		break;
	case RDTS_N_RIGHT:
		if(!this->pSprites[uiSelSpriteVirt]->draw(relX,relY) ||
		!this->pSprites[uiSelSpriteVirt]->draw(relX+RDTG_XOFF,relY) ||
		!this->pSprites[uiSelSpriteVirt]->draw(relX,relY+RDTG_YOFF) ||
		!this->pSprites[uiSelSpriteVirt]->draw(relX-RDTG_XOFF,relY+RDTG_YOFF)) return false;
		break;
	default:
		return false;
		break;
	}
	return true;
}//bool	RDTetris::DrawNext(int relX,int relY)


///////////////////////////
//External help functions//
///////////////////////////



bool		RDTLoadGraphics(char Path[],RDTetris *pTarget){ //Loads the Graphic set defined in the scriptfile
	char	LoadTmp[100];
	char	Action[100];
	int		Target=0;
	FILE	*pFile=NULL;
	freeSprite *pNewSprite=NULL;

	if(pTarget==NULL) return false;

	pFile=fopen(Path,"r");
	if(pFile==NULL) return false;


	fgets(LoadTmp,100,pFile);
	if(strcmp(LoadTmp,"RDTLoadGraphicsScriptFile\n")!=0) {
		printf("%s\n", LoadTmp);
		fclose(pFile);
		return false;
	}

	while(fscanf(pFile,"%s %i %s\n",Action,&Target,LoadTmp)!=EOF) {
		if(feof(pFile)) return true;
		if(strcmp(Action,"SetGraphic")!=0) {
			while(fgetc(pFile)!='\n') {};
			continue;
		}


		if(Target<(-1) || Target>RDTG_SPRITES) continue;
		
		pNewSprite	=new freeSprite;
		if(pNewSprite==NULL) {
			fclose(pFile);
			return false;
		}

		if(!pNewSprite->init(LoadTmp,RDTG_CK)) {
			delete pNewSprite;
			pNewSprite=NULL;
			fclose(pFile);
			return false;
		}
		if(pTarget->GetGraphic(Target)!=NULL) {
			delete pNewSprite;
			pNewSprite=NULL;
			fclose(pFile);
			return false;
		}
		if(!pTarget->SetGraphic(Target,pNewSprite)) {
			delete pNewSprite;
			pNewSprite=NULL;
			fclose(pFile);
			return false;
		}

		pNewSprite=NULL;
	}
	return true;
}//bool		RDTLoadGraphics(char Path[])


bool		RDTLoadGraphicsB(char Path[],RDTetris *pTarget){	//Loads the Graphic set defined by the name Path1.jpg ,Path2.jpg s.o.
	char			TmpPath[100];
	char			Ending[20];
	char			MPath[100];
	freeSprite		*pNewSprite=NULL;
	int				Counter=(-1);

	if(pTarget==NULL) return false;
	
	sscanf(Path,"%s.%s",MPath,Ending);

	while(1) {
		pNewSprite	=new freeSprite;
		if(pNewSprite==NULL) return false;
		
		sprintf(TmpPath,"%s%i.%s",MPath,Counter,Ending);


		if(!pNewSprite->init(TmpPath,RDTG_CK)) return false;

		
		if(pTarget->GetGraphic(Counter)!=NULL) {
			delete pNewSprite;
			pNewSprite=NULL;
			return false;
		}

		if(!pTarget->SetGraphic(Counter,pNewSprite)) {
			delete pNewSprite;
			pNewSprite=NULL;
			return false;
		}

		pNewSprite=NULL;

		Counter++;	
		if(Counter>RDTG_SPRITES) return true;
		
	}
	return true;
}//bool		RDTLoadGraphicsB(char Path[],RDTetris *pTarget)


void		RDTUnloadGraphics(RDTetris *pTarget){				//Unloads all Graphics
	freeSprite	*pDel=NULL;
	if(pTarget==NULL) return;

	for(int i=(-1);i<=RDTG_SPRITES;i++) {
		pDel=pTarget->GetGraphic(i);
		if(pDel==NULL) continue;
		delete pDel;
		pDel=NULL;
		pTarget->SetGraphic(i,NULL);
	}
	return;
}//void		RDTUnloadGraphics(RDTetris *pTarget)




/****************************************/
/*		By RedDesert (z-man) 2003C		*/
/*		  Please visit our page:		*/
/*			www.RedDesert.de			*/
/****************************************/

//last changes: tijot 0.96.a - zman

