/****************************************/
/*				RDTetris.h				*/
/* A Tetris Engine by RedDesert, you    */
/* may redistribute this source at your */
/* will! But please send us a mail at   */
/* info@reddesert.de or post in our     */
/* board at www.reddesert.de!!			*/
/****************************************/
/*			www.RedDesert.de			*/
/****************************************/
/*	Project start:			12.08.03	*/
/*	Project last change:	20.08.03	*/
/****************************************/
/*	ToDo:								*/
/*		-Nothing						*/
/****************************************/

#ifndef		RDTETRIS_H
#define		RDTETRIS_H

/*Constants*/


//Game Constants
#define		RDTG_DELAY				1000//RDTG_DELAY - Given Speed = Real Delay
#define		RDTG_CONTROLLERDELAY	200	//Defines speed of the input
#define		RDTG_PPL				10	//Points per Line
#define		RDTG_PPS				0	//Points per Stone
#define		RDTG_SUBMULTI			4	//Speedupmultiplicator

//Action Flags
#define		RDTA_NOTHING		0x00	//Does nothing
#define		RDTA_LEFT			0x02	//Moves the stone left
#define		RDTA_RIGHT			0x04	//Moves the stone right
#define		RDTA_ROTATE			0x08	//Rotates the stone
#define		RDTA_SPEEDUP		0x10	//Activates the Speed up

//Field Constants
#define		RDTF_WIDTH			10	//Width of the field
#define		RDTF_HEIGHT			20	//Height of the field
#define		RDTF_STARTX			RDTF_WIDTH/2-1   //Start coordinate x
#define		RDTF_STARTY			0	//Start coordinate y


//Stone Constants
#define		RDTS_QUADSTONE		3	//Number of Quads in a stone
#define		RDTS_STONEC			8	//Number of integer variables for Stone quad coordinates
#define		RDTS_COUNT			7	//Number of stones

#define		RDTS_QUAD			0	//A Quadratic stone
#define		RDTS_LINE			1	//A Line-Shaped stone
#define		RDTS_L_LEFT			2	//A left orientated L-Shaped stone
#define		RDTS_L_RIGHT		3	//A right orientated L-Shaped stone
#define		RDTS_N_LEFT			4
#define		RDTS_N_RIGHT		5
#define		RDTS_T				6	//A T-Shaped stone

//RedDesert Tetris Flags
#define		RDTF_STANDART		0x00	//Nothing
#define		RDTF_RUNNING		0x02	//Game's running
#define		RDTF_LOST			0x04	//Game's lost
#define		RDTF_STONE			0x08	//A Stone is falling
#define		RDTF_PAUSED			0x10	//Game's paused
#define		RDTF_SPEEDUP		0x20	//SpeedUp is activated

#define		RDTF_SCRIPTEND		0x40	//Marks the end of the script file

//RedDesert Tetris Quad Flags
#define		RDTF_LOCKED			0x02	//Quad is locked
#define		RDTF_DESTROYED		0x04	//Quad is destroyed


//RedDesert Tetris Script defines
#define		RDTS_HEADER			"RDTetrisScriptFile"
#define		RDTS_ADRESS			"www.RedDesert.de"
#define		RDTS_END			"EndOfScriptFile"

#define		RDTS_ACTION			"Action="
#define		RDTS_DELAY			"Delay="
#define		RDTS_NEXTTYPE		"NextType="
#define		RDTS_COLOUR			"Colour="

//Grafic defines
#define		RDTG_SPRITES		1 
#define		RDTG_CK				0xFF00FF

#define		RDTG_XOFF			22
#define		RDTG_YOFF			22

//Sound defines
#define		RDTM_STONEDOWN		0
#define		RDTM_CRACKLINE		1
#define		RDTM_COUNT			1

//Network defines
//#define		RDTN_ENABLE			//If not defined network capabilities will be disabled

#define		RDTN_PORT			6666		//Default Tijot netzwerk port
#define		RDTN_VERSION		0			//Version of the network device
#define		RDTN_ID				"RedDesertTijotNetwork" //ID of our game

#define		RDTN_STANDART		0
#define		RDTN_SERVER			1
#define		RDTN_CLIENT			2
#define		RDTN_CNT			2

//Compiler switches
#ifdef RDTN_ENABLE
#ifdef _WIN32			//Switch includes for network device
#include <winsock2.h>
#else
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#endif
#endif
/*Includes*/

#include <stdio.h>
#include <string.h>
#include "../tfree/tfree.h"	//TFree Graphic class

/*Class&Struct Predefine*/

//Structs Pre.

struct	RDQUAD;		//Quad structure for single elements of the stones
struct	RDNPacket;	//Transmitted packet

//Classes Pre.

class	RDTetris;	//Tetris class to handle everything*g*

#ifdef RDTN_ENABLE
class	RDTetrisNetwork;//Net playing class
#endif

	
//Structs

typedef struct RDQUAD{	//Quad structure for singe elements of the stones
	unsigned char		uchFlags;	//Flag memory
	unsigned char		uchSprite;	//Number of the Used Sprite in the Sprite array
}RDQUAD,*PRDQUAD;

//Classes

class RDTetris{			//Complete Tetris steering class
private:
	
	//Field
	RDQUAD				*pField[RDTF_WIDTH+1][RDTF_HEIGHT+1]; //Pointer array 
	//Stone
	RDQUAD				*pStone[RDTS_QUADSTONE+1];			//Pointer to Stone Elements
	int					iaStoneCoord[RDTS_STONEC+1];	//Integer Array to Save Stone Quad Data
	int					iType;			//Actual Stone Type
	int					iNextType;		//Next Stone Type
	
	//Game Counter
	int					iPoints;		//Number of Points actually reached
	int					iLines;			//Number of cracked lines
	unsigned int		uiDelay;		//Delay of wait, is worked out by the given speed value
	unsigned char		uchFlags;		//Flag memory
	unsigned int		uiTimer;		//Timer for Movement
	unsigned int		uiITimer;		//Timer for Input
	unsigned int		uiSelSprite;	//Selected Sprite

	int					iLPoints;		//Last number of points (is refreshed in every Run call)
	int					iLLines;		//Last number of lines	(is refreshed in every Run call);


	//Grafic
	freeSprite			*pSprites[RDTG_SPRITES+1];	//Sprite Array
	freeSprite			*pFieldGrafic;				//Field grafic

	//Sound
	CSound				*pSounds[RDTM_COUNT+1];		//Sound array


	//Internal functions
	inline bool		IsTouched(int y);		//Is line y touched by a quad?
	inline bool		IsBound(int x,int y);	//Is already a quad at pos x,y

	inline void		CrackLine(int y);		//Kill all quads in line y
	inline void		UnlockField(int y);		//Unlocks all Quads over the y coord
	
	inline bool		TransformXY(int difx,int dify);//Transforms actual stone on x and y axes
	inline bool		TransformRot(void);	//Transforms actual stone with rotate

	inline void		LockStone(void);		//Lock stone so the engine doesn't move it
	inline bool		SetStone(int Colour);	//Sets stone by using the iaStone Coords

	inline void		Move(void);				//Moves all unlocked stones down and locks them

public:

	RDTetris(void);		//Constructor
	~RDTetris(void);	//Destructor
	
	bool			Draw(int relX,int relY);				//Draws the whole field
	bool			DrawActual(int relX,int relY);			//Draws the actual Stone
	bool			DrawNext(int relX,int relY);			//Draws the next Stone

	//Sound
	bool			SetSound(int ArrayIndex,CSound	*pPointer);
	CSound			*GetSound(int ArrayIndex);

	//Graphic
	bool			SetGraphic(int ArrayIndex,freeSprite *pPointer); //Sets a Graphic into the class array
	freeSprite *	GetGraphic(int ArrayIndex);	//Returns the graphic value set by the user

	bool			Init(int iSpeed);	//Init Game
	bool			Run(int Action);	//Run game and perform Action
	void			Reset(void);		//Reset Game and Release all used memory
	void			Restart(int iSpeed);//Resets only the game parts and makes them ready for a new game

	bool		Lost(void);			//Checks if player has lost or not
	int			GetPoints(void);	//Returns number of won points
	void		SetPoints(int iPoints);//Sets Points
	int			GetLines(void);		//Returns number of cracked lines
	void		SetLines(int iLines);//Sets Points
	int			GetSpeed(void);		//Returns the given speed value
	int			GetStoneType(void);	//Returns the ID of the actual falling stone
	int			GetNextStoneType(void);//Returns the ID if the next falling stone
	int			GetLastPoints(void); //Returns amount of points earned in the last run call
	int			GetLastLines(void);	//Returns amount of lines cracked in the last run call
	inline bool	IsFilled(int y);	//Is line y completely filled with quads?
	void		SetSpeed(int iSpeed);//Sets a new speed value
};//class RDTetris











///////////////////////////
//External help functions//
///////////////////////////

bool		RDTLoadGraphics(char Path[],RDTetris *pTarget);		//Loads the Graphic set defined in the scriptfile
bool		RDTLoadGraphicsB(char Path[],RDTetris *pTarget);	//Loads the Graphic set defined by the name Path1.jpg ,Path2.jpg s.o.
void		RDTUnloadGraphics(RDTetris *pTarget);				//Unloads all Graphics




#endif
/****************************************/
/*		By RedDesert (z-man) 2003C		*/
/*		  Please visit our page:		*/
/*			www.RedDesert.de			*/
/****************************************/

//last changes: tijot 0.96.a - zman

